#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function
from six import string_types



from libsignetsim.model.math.sympy_shortcuts import *
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.MathException import MathException

import libsbml
from sympy import srepr
from re import match


class SbmlMathWriter(object):
	""" Class for handling math formulaes """

	def __init__(self, model):
		""" Constructor """

		self.model = model


	def writeSbml(self, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Export math formula to sbml """

		formula = self.translateForSbml(self.getInternalMathFormula(), sbml_level, sbml_version)
		if Settings.verbose >= 2:
			print("\n> writeSbml")
			print(">> input : %s" % srepr(self.getInternalMathFormula()))
			print(">> output : %s" % self.printSbml(formula, sbml_level, sbml_version))

		return formula

	def printSbml(self, formula, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		if isinstance(formula, string_types):
			return formula
		elif isinstance(formula, libsbml.ASTNode):
			if sbml_level <= 2:
				return libsbml.formulaToString(formula)
			else:
				return libsbml.formulaToL3String(formula)
		else:
			return str(formula)

	def translateVariableForSbml(self, variable, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Translates a Sympy symbol in C """

		#Input is a Sympy symbol, we need to convert to string
		variable = str(variable)

		if variable == "_time_":
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_NAME_TIME)
			t_ast.setName("time")
			return t_ast

		elif variable == "_avogadro_":
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_NAME_AVOGADRO)
			return t_ast


		if variable.startswith("_speciesForcedConcentration_"):
			res_match = match(r"_speciesForcedConcentration_(.*)_", variable)
			t_sbml_id = str(res_match.groups()[0])
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_NAME)
			t_ast.setName(self.model.listOfSpecies.getBySbmlId(t_sbml_id).getSbmlId())
			return t_ast

		elif "_speciesForcedAmount_" in variable:
			res_match = match(r"_speciesForcedAmount_(\d+)_", variable)
			t_id = int(res_match.groups()[0])
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_NAME)
			t_ast.setName(self.model.listOfSpecies[t_id].getSbmlId())
			return t_ast

		elif "_local_" in variable:
			res_match = match(r"_local_(\d+)_(.*)", variable)
			t_rid = int(res_match.groups()[0])
			t_sbmlid = str(res_match.groups()[1])
			t_localparam = self.model.listOfReactions[t_rid].listOfLocalParameters.getBySbmlId(t_sbmlid)
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_NAME)
			t_ast.setName(t_localparam.getSbmlId())
			return t_ast

		elif "_functionDefinition_" in str(variable):
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION)

			res_match = match(r"_functionDefinition_(\d+)_", str(variable))
			t_id = int(res_match.groups()[0])

			t_ast.setName(self.model.listOfFunctionDefinitions[t_id].getSbmlId())
			return t_ast

		else:
			# print "translateVariableToSbml unknown variable type ! (%s)" % variable
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_NAME)
			t_ast.setName(variable)
			return t_ast


	def translateForSbml(self, tree, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Translate a sympy tree into a C string """

		# if isinstance(tree, int):
		# 	t_ast = libsbml.ASTNode()
		# 	t_ast.setType(libsbml.AST_INTEGER)
		# 	t_ast.setValue(tree)
		# 	return t_ast
		#
		# elif isinstance(tree, float):
		# 	t_ast = libsbml.ASTNode()
		# 	t_ast.setType(libsbml.AST_REAL)
		# 	t_ast.setValue(tree)
		# 	return t_ast

		# elif isinstance(tree, str):
		# 	return self.translateVariableForSbml(tree, sbml_level, sbml_version)

		if tree is None:
			return None

		elif tree.func == SympyTrue:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_CONSTANT_TRUE)
			return t_ast

		elif tree.func == SympyFalse:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_CONSTANT_FALSE)
			return t_ast

		elif tree.func == SympySymbol or tree.func == SympyDummy:
			return self.translateVariableForSbml(str(tree), sbml_level, sbml_version)

		# elif isinstance(tree.func, SympyUndefinedFunction) and tree.args == (SympySymbol("t"),) and str(tree.func) in self.model.listOfVariables.keys():
		# 	return self.translateVariableForSbml(str(tree.func), sbml_level, sbml_version)

		elif tree.func == SympyInteger:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_INTEGER)
			t_ast.setValue(int(tree))
			return t_ast

		elif tree.func == SympyFloat:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_REAL)
			t_ast.setValue(float(tree))
			return t_ast


		elif tree.func == SympyRational:
			# One of the values is a float...
			# The python libsbml doesn't seems to be implemented in that case.
			# Not sure if it's standard or not, but the test case exists... so
			if float(int(tree.p)) != float(tree.p) or float(int(tree.p)) != float(tree.p):
				t_ast = libsbml.ASTNode()
				t_ast.setType(libsbml.AST_DIVIDE)
				t_ast.addChild(self.translateForSbml(tree.p, sbml_level, sbml_version))
				t_ast.addChild(self.translateForSbml(tree.q, sbml_level, sbml_version))
				return t_ast


			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_RATIONAL)
			t_ast.setValue(int(tree.p), int(tree.q))
			return t_ast

		elif tree.func == SympyNegOne:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_INTEGER)
			t_ast.setValue(-1)
			return t_ast

		elif tree.func == SympyOne:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_INTEGER)
			t_ast.setValue(1)
			return t_ast

		elif tree.func == SympyHalf:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_REAL)
			t_ast.setValue(0.5)
			return t_ast

		elif tree.func == SympyZero:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_INTEGER)
			t_ast.setValue(0)
			return t_ast

		elif tree == SympyPi:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_CONSTANT_PI)
			return t_ast

		elif tree.func == SympyE or tree.func == SympyExp1:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_CONSTANT_E)
			return t_ast

		elif tree == SympyInf:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_REAL)
			t_ast.setValue(float("inf"))
			return t_ast

		elif tree == -SympyInf:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_REAL)
			t_ast.setValue(float("-inf"))
			return t_ast

		elif tree == SympyNan:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_REAL)
			t_ast.setValue(float("nan"))
			return t_ast

		elif tree == SympyTrue or tree == True:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_CONSTANT_TRUE)
			return t_ast

		elif tree == SympyFalse or tree == False:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_CONSTANT_FALSE)
			return t_ast

		elif tree.func == SympyAdd:

			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_PLUS)
			for i_arg, arg in enumerate(tree.args):
				t_ast.addChild(self.translateForSbml(arg, sbml_level, sbml_version))

			return t_ast

		elif tree.func == SympyMul:

			if len(tree.args) == 2:
				if tree.args[0].func == SympyNegOne:
					t_ast = libsbml.ASTNode()
					t_ast.setType(libsbml.AST_MINUS)
					t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
					return t_ast

				if tree.args[1].func == SympyNegOne:
					t_ast = libsbml.ASTNode()
					t_ast.setType(libsbml.AST_MINUS)
					t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
					return t_ast

			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_TIMES)
			# t_ast_2 = None
			t_tree_denominator = None
			t_tree_numerator = None

			for i_arg, arg in enumerate(tree.args):
				if arg.func == SympyPow and arg.args[1].func == SympyNegOne:

					if t_tree_denominator is None:
						t_tree_denominator = self.translateForSbml(arg.args[0], sbml_level, sbml_version)
					elif t_tree_denominator.getType() != libsbml.AST_TIMES:
						t_tree_denominator_new = libsbml.ASTNode()
						t_tree_denominator_new.setType(libsbml.AST_TIMES)
						t_tree_denominator_new.addChild(t_tree_denominator)
						t_tree_denominator_new.addChild(self.translateForSbml(arg.args[0], sbml_level, sbml_version))
						t_tree_denominator = t_tree_denominator_new
					else:
						t_tree_denominator.addChild(self.translateForSbml(arg.args[0], sbml_level, sbml_version))
				else:
					t_tree_numerator = self.translateForSbml(arg, sbml_level, sbml_version)
					# t_ast.addChild(t_tree_numerator)
					t_ast.addChild(self.translateForSbml(arg, sbml_level, sbml_version))

			if t_tree_denominator is not None:
				t_ast_2 = libsbml.ASTNode()
				t_ast_2.setType(libsbml.AST_DIVIDE)
				if t_ast.getNumChildren() == 1:
					t_ast_2.addChild(t_tree_numerator)
				else:
					t_ast_2.addChild(t_ast)
				t_ast_2.addChild(t_tree_denominator)
				return t_ast_2

			return t_ast

		# rateOf
		elif tree.func == SympyRateOf:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_RATE_OF)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# quotient
		elif tree.func == SympyQuotient:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_QUOTIENT)
			# t_ast.setName("quotient")
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		# quotient
		elif tree.func == SympyRem:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_REM)
			# t_ast.setName("rem")
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		# implies
		elif tree.func == SympyImplies:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_LOGICAL_IMPLIES)
			# t_ast.setName("implies")
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		# min
		elif tree.func == SympyUnevaluatedMin:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_MIN)
			for i_arg, arg in enumerate(tree.args):
				t_ast.addChild(self.translateForSbml(arg, sbml_level, sbml_version))

			return t_ast

		# max
		elif tree.func == SympyUnevaluatedMax:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_MAX)
			for i_arg, arg in enumerate(tree.args):
				t_ast.addChild(self.translateForSbml(arg, sbml_level, sbml_version))

			return t_ast

		# AST_FUNCTION_ABS
		elif tree.func == SympyAbs:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_ABS)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_ARCCOS
		elif tree.func == SympyAcos:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_ARCCOS)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_ARCCOSH
		elif tree.func == SympyAcosh:
			if tree.args[0].func == SympyPow and tree.args[0].args[1] == SympyInteger(-1):
				t_ast = libsbml.ASTNode()
				t_ast.setType(libsbml.AST_FUNCTION_ARCSECH)
				t_ast.addChild(self.translateForSbml(tree.args[0].args[0], sbml_level, sbml_version))
				return t_ast

			else:
				t_ast = libsbml.ASTNode()
				t_ast.setType(libsbml.AST_FUNCTION_ARCCOSH)
				t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
				return t_ast

		# AST_FUNCTION_ARCCOT
		elif tree.func == SympyAcot:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_ARCCOT)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_ARCCOTH
		elif tree.func == SympyAcoth:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_ARCCOTH)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_ARCSIN
		elif tree.func == SympyAsin:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_ARCSIN)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_ARCSINH
		elif tree.func == SympyAsinh:
			if tree.args[0].func == SympyPow and tree.args[0].args[1] == SympyInteger(-1):
				t_ast = libsbml.ASTNode()
				t_ast.setType(libsbml.AST_FUNCTION_ARCCSCH)
				t_ast.addChild(self.translateForSbml(tree.args[0].args[0], sbml_level, sbml_version))
				return t_ast
			else:
				t_ast = libsbml.ASTNode()
				t_ast.setType(libsbml.AST_FUNCTION_ARCSINH)
				t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
				return t_ast

		# AST_FUNCTION_ARCTAN
		elif tree.func == SympyAtan:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_ARCTAN)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_ARCTANH
		elif tree.func == SympyAtanh:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_ARCTANH)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_ARCSEC
		elif tree.func == SympyAsec:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_ARCSEC)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_ARCCSC
		elif tree.func == SympyAcsc:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_ARCCSC)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_CEILING
		elif tree.func == SympyCeiling:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_CEILING)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_COS
		elif tree.func == SympyCos:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_COS)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_COSH
		elif tree.func == SympyCosh:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_COSH)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_COT
		elif tree.func == SympyCot:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_COT)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_COTH
		elif tree.func == SympyCoth:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_COTH)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_CSC
		elif tree.func == SympyCsc:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_CSC)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_DELAY
		# TODO
		#
		# AST_FUNCTION_EXP
		elif tree.func == SympyExp:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_EXP)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_FACTORIAL
		elif tree.func == SympyFactorial:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_FACTORIAL)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_FLOOR
		elif tree.func == SympyFloor:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_FLOOR)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_LOG
		#TODO
		elif tree.func == SympyLog:
			if len(tree.args) < 2:
				t_ast = libsbml.ASTNode()
				t_ast.setType(libsbml.AST_FUNCTION_LN)
				t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
				return t_ast

			else:
				if tree.args[1] == SympyInteger(10):
					t_ast = libsbml.ASTNode()
					t_ast.setType(libsbml.AST_FUNCTION_LOG)

					t_ast_2 = libsbml.ASTNode()
					t_ast_2.setType(libsbml.AST_INTEGER)
					t_ast_2.setValue(10)

					t_ast.addChild(t_ast_2)
					t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
					return t_ast
				else:
					t_ast = libsbml.ASTNode()
					t_ast.setType(libsbml.AST_FUNCTION_LOG)
					t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
					t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
					return t_ast

		# AST_FUNCTION_PIECEWISE
		elif tree.func == SympyPiecewise:

			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_PIECEWISE)

			(t_val, t_cond) = tree.args[0]

			t_ast.addChild(self.translateForSbml(t_val, sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(t_cond, sbml_level, sbml_version))
			last_child = []
			for piece in range(1, len(tree.args)):
				(t_val, t_cond) = tree.args[piece]

				if piece == (len(tree.args)-1):
					t_ast.addChild(self.translateForSbml(t_val, sbml_level, sbml_version))
				else:
					t_ast.addChild(self.translateForSbml(t_val, sbml_level, sbml_version))
					t_ast.addChild(self.translateForSbml(t_cond, sbml_level, sbml_version))

			return t_ast

		elif tree.func == SympyITE:

			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_PIECEWISE)

			t_cond = tree.args[0]
			t_val = tree.args[1]
			t_val_else = tree.args[2]
			t_ast.addChild(self.translateForSbml(t_val, sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(t_cond, sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(t_val_else, sbml_level, sbml_version))
			return t_ast

		#TODO
		# AST_FUNCTION_POWER
		elif tree.func == SympyPow:

			# A standard 1/b division will be written mul(1, pow(b,-1))
			# We recognize it and write it as a division
			# Important to note that a/b is written mul(a, mul(1, pow(b,-1)))
			# so we don't need to look for a
			if tree.args[1].func == SympyNegOne:
				t_ast = libsbml.ASTNode()
				t_ast.setType(libsbml.AST_DIVIDE)

				t_ast_2 = libsbml.ASTNode()
				t_ast_2.setType(libsbml.AST_INTEGER)
				t_ast_2.setValue(1)

				t_ast.addChild(t_ast_2)
				t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
				return t_ast

			# Similar for SQRT(x), it's written pow(x, pow(2, -1))
			elif tree.args[1].func == SympyPow and tree.args[1].args[0] == SympyInteger(2) and tree.args[1].args[1].func == SympyNegOne:
				t_ast = libsbml.ASTNode()
				t_ast.setType(libsbml.AST_FUNCTION_ROOT)

				t_ast_2 = libsbml.ASTNode()
				t_ast_2.setType(libsbml.AST_INTEGER)
				t_ast_2.setValue(2)

				t_ast.addChild(t_ast_2)
				t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
				return t_ast

			# print "\n"
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_POWER)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		# #TODO
		# # AST_FUNCTION_ROOT
		# elif tree.func == SympyRoot:
		#     print "> Root case... Not done yet. Truth is I don't think it's used..."
		#     # t_ast = libsbml.ASTNode()
		#     # t_ast.setType(libsbml.AST_POWER)
		#     return "rt_pow(" + self.translateForSbml(tree.args[0], sbml_level, sbml_version) + ",((realtype)1)/" + self.translateForSbml(tree.args[1], sbml_level, sbml_version) + ")"

		# AST_FUNCTION_SEC
		elif tree.func == SympySec:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_SEC)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_SIN
		elif tree.func == SympySin:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_SIN)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_SINH
		elif tree.func == SympySinh:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_SINH)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_TAN
		elif tree.func == SympyTan:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_TAN)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		# AST_FUNCTION_TANH
		elif tree.func == SympyTanh:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION_TANH)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyEqual:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_RELATIONAL_EQ)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyUnequal:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_RELATIONAL_NEQ)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyGreaterThan:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_RELATIONAL_GEQ)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyLessThan:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_RELATIONAL_LEQ)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyStrictGreaterThan:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_RELATIONAL_GT)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyStrictLessThan:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_RELATIONAL_LT)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyAnd:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_LOGICAL_AND)
			for i_arg in range(0, len(tree.args)):
				t_ast.addChild(self.translateForSbml(tree.args[i_arg], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyOr:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_LOGICAL_OR)
			for i_arg in range(0, len(tree.args)):
				t_ast.addChild(self.translateForSbml(tree.args[i_arg], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyXor:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_LOGICAL_XOR)
			for i_arg in range(0, len(tree.args)):
				t_ast.addChild(self.translateForSbml(tree.args[i_arg], sbml_level, sbml_version))
			return t_ast

		elif tree.func == SympyNot:
			# print tree
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_LOGICAL_NOT)
			t_ast.addChild(self.translateForSbml(tree.args[0], sbml_level, sbml_version))
			return t_ast
		elif tree.func == SympyLambda or tree.func == SympyIdentityFunction:
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_LAMBDA)
			t_args = list(tree.args[0])
			for arg in t_args:
				t_ast.addChild(self.translateForSbml(arg, sbml_level, sbml_version))
			t_ast.addChild(self.translateForSbml(tree.args[1], sbml_level, sbml_version))
			return t_ast

		elif "_functionDefinition_" in str(tree.func):
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION)

			res_match = match(r"_functionDefinition_(\d+)_", str(tree.func))
			t_id = int(res_match.groups()[0])

			t_ast.setName(self.model.listOfFunctionDefinitions[t_id].getSbmlId())
			for i_arg in range(0, len(tree.args)):
				# print tree.args[i_arg]
				t_ast.addChild(self.translateForSbml(tree.args[i_arg], sbml_level, sbml_version))

			return t_ast

		elif "_functionDefinition_" in str(tree):
			t_ast = libsbml.ASTNode()
			t_ast.setType(libsbml.AST_FUNCTION)

			res_match = match(r"_functionDefinition_(\d+)_", str(tree))
			t_id = int(res_match.groups()[0])

			t_ast.setName(self.model.listOfFunctionDefinitions[t_id].getSbmlId())
			# for i_arg in range(0, len(tree.args)):
			# 	# print tree.args[i_arg]
			# 	t_ast.addChild(self.translateForSbml(tree.args[i_arg], sbml_level, sbml_version))

			return t_ast
		else:
			# print str(tree)
			raise MathException("Sbml Math Writer : Unknown Sympy Symbol %s" % str(tree))
			return str(tree)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from numpy import amin, amax, linspace, interp
from libsignetsim.data.ExperimentalData import ExperimentalData

class ListOfExperimentalData(dict):

	def __init__(self):

		self.currentId = 0


	def add(self, experimental_data):
		self.update({self.currentId: experimental_data})
		self.currentId += 1

	def readNuML(self, list_of_data):

		for data in list_of_data.getContents():
			exp_data = ExperimentalData()
			exp_data.readNuML(data)
			self.add(exp_data)

	def writeNuML(self, composite_value):

		for data in list(self.values()):
			data.writeNuML(composite_value)

	def getMaxTime(self):

		max_time = 0
		for data in list(self.values()):
			if data.t > max_time:
				max_time = data.t
		return max_time

	def getSpecies(self):
		""" Returns an array of sbml ids"""
		species = []
		for data in list(self.values()):
			species.append(data.name)
		return list(set(species))

	def getTimes(self):

		times = []
		for data in list(self.values()):
			times.append(data.t)

		return times


	def getByVariable(self):
		result = {}
		for species in self.getSpecies():
			result.update({species: []})
		for data in list(self.values()):
			result[data.name].append(data)

		return result


	def getValuesOfSpecies(self):

		values = {}
		for data in list(self.values()):
			if data.name not in list(values.keys()):
				values.update({data.name: []})

			values[data.name].append(data.value)
		return values

	def getTimesOfSpecies(self):

		times = {}
		for data in list(self.values()):
			if data.name not in list(times.keys()):
				times.update({data.name: []})

			times[data.name].append(data.t)

		return times

	def getValues(self):

		values = {}
		for data in list(self.values()):
			if data.name not in list(values.keys()):
				values.update({data.name: []})

			values[data.name].append((data.t, data.value))

		return values

	def interpolate(self, size=101):

		new_experimental_data = {}
		new_currentId = 0

		list_species = list(set([data.name for data in list(self.values())]))

		for species in list_species:
			times = []
			values = []
			for data in list(self.values()):
				if data.name == species:
					times.append(data.t)
					values.append(data.value)

			if amin(times) != amax(times):
				times_interpolation = linspace(amin(times), amax(times), size)
				values_interpolation = interp(times_interpolation, times, values)

				for i_data, data in enumerate(values_interpolation):
					t_data = ExperimentalData()
					t_data.readDB(species, times_interpolation[i_data], data)
					new_experimental_data.update({new_currentId: t_data})
					new_currentId += 1
			else:
				t_data = ExperimentalData()
				t_data.readDB(species, times[0], values[0])
				new_experimental_data.update({new_currentId: t_data})
				new_currentId += 1

		dict.clear(self)
		dict.update(self, new_experimental_data)
		self.currentId = new_currentId

	def getVariables(self):

		variables = []
		for data in list(self.values()):
			variables.append(data.name)

		return list(set(variables))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Initialization of the combine archive module

"""

from .Manifest import Manifest
from .File import File
from libsignetsim.settings.Settings import Settings
from .CombineException import NoManifestFoundException

class ListOfFiles(list):

	MANIFEST = "manifest.xml"

	def __init__(self, archive):

		list.__init__(self)
		self.__archive = archive
		self.__manifest = Manifest(self.__archive)


	def readListOfFiles(self, archive_file):

		if not self.MANIFEST in archive_file.namelist():
			# raise NoManifestFoundException("No manifest found")
			pass
		else:
			# with archive_file.read(self.MANIFEST) as manifest_file:
			# 	self.__manifest.readManifest(manifest_file)
			self.__manifest.readManifest(archive_file.read(self.MANIFEST))
		# self.__manifest.writeManifest()

		for t_file in archive_file.namelist():
			if not t_file.endswith("/") and t_file != self.MANIFEST:
				file = File(self.__archive, self.__manifest)
				file.readFile(t_file, archive_file)
				list.append(self, file)

		if self.getMasterSedml() is None and self.__archive.isSEDMLArchive():
			if len(self.getAllSedmls()) > 0:
				new_master = self.getAllSedmls()[0]
				new_master.setMaster()

	def addFile(self, filename):

		file = File(self.__archive, self.__manifest)
		file.newFile(filename)
		list.append(self, file)
		return file

	def writeManifest(self):
		return self.__manifest.writeManifest()

	def writeListOfFiles(self):
		return [file.getFilename() for file in self]

	# def getManifest(self):
	# 	return self.__manifest

	def getMasterSedml(self):
		for file in self:
			if file.isMaster() and file.isSedml():
				return file

	def getAllSedmls(self):
		return [file for file in self if file.isSedml()]

	def getMasterSbml(self):
		for file in self:
			if file.isMaster() and file.isSbml():
				return file

	def getAllSbmls(self):
		return [file for file in self if file.isSbml()]

	def getAllNumls(self):
		return [file for file in self if file.isNuml()]

	def getMasterNuml(self):
		for file in self:
			if file.isMaster() and file.isNuml():
				return file



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.ModelInstance import ModelInstance
from libsignetsim.settings.Settings import Settings
from libsignetsim.optimization.Optimization import Optimization
from libsignetsim.optimization.NoiseGenerator import NoiseGenerator
from libsignetsim.cwriter.CWriterModelVsDataOptimization import CWriterModelVsDataOptimization


class ModelVsTimeseriesOptimization(Optimization, CWriterModelVsDataOptimization, NoiseGenerator):

	def __init__(
			self, workingModel, parameters_to_fit=None,
			list_of_experiments=None, reference_data=None,
			mapping=None, noise=0, sampling=None, nb_procs=1,
			p_lambda=Settings.defaultPlsaLambda,
			p_criterion=Settings.defaultPlsaCriterion,
			p_precision=Settings.defaultPlsaPrecision,
			p_initial_temperature=Settings.defaultPlsaInitialTemperature,
			p_gain=Settings.defaultPlsaGainForJumpSizeControl,
			p_interval=Settings.defaultPlsaInterval,
			p_mix=Settings.defaultPlsaMixInterval,
			p_initial_moves=Settings.defaultPlsaInitialMoves,
			p_tau=Settings.defaultPlsaTau,
			p_freeze_count=Settings.defaultPlsaFreezeCount,
			s_neg_penalty=Settings.defaultScoreNegativePenalty,
	):

		self.compModelDefinition = None

		if workingModel.parentDoc.isCompEnabled() and not isinstance(workingModel, ModelInstance):
			self.compModelDefinition = workingModel
			self.workingModel = workingModel.parentDoc.getModelInstance()
			self.parameters = []
			for parameter, init_val, lower_bound, upper_bound, precision in parameters_to_fit:
				self.parameters.append((
					self.workingModel.getInstanceVariableByXPath(parameter.getXPath()), init_val, lower_bound, upper_bound, precision
				))

		else:

			self.workingModel = workingModel
			self.parameters = []
			for parameter, init_val, lower_bound, upper_bound, precision in parameters_to_fit:
				self.parameters.append((
					workingModel.parentDoc.getByXPath(parameter.getXPath()), init_val, lower_bound, upper_bound, precision
				))

			self.workingModel = workingModel

		if list_of_experiments is not None:
			self.listOfExperiments = list_of_experiments

		Optimization.__init__(
			self,
			workingModel=self.workingModel,
			parameters_to_fit=self.parameters,
			optimization_type=Optimization.MODEL_VS_DATA,
			model_instance=(self.compModelDefinition is not None)
		)

		CWriterModelVsDataOptimization.__init__(
			self, self.workingModel, self.listOfExperiments, mapping, self.parameters,
			p_lambda=p_lambda, p_criterion=p_criterion, p_precision=p_precision,
			p_initial_temperature=p_initial_temperature, p_gain=p_gain, p_interval=p_interval, p_mix=p_mix,
			p_initial_moves=p_initial_moves, p_tau=p_tau, p_freeze_count=p_freeze_count, s_neg_penalty=s_neg_penalty
		)
		NoiseGenerator.__init__(self, self.listOfExperiments, noise, sampling)

		self.mapping = mapping
		self.noise = noise
		self.sampling = sampling


	def writeOptimizationFiles(self, nb_procs=1):

		Optimization.writeOptimizationFilesMain(self)
		CWriterModelVsDataOptimization.writeOptimizationFiles(self, nb_procs)


	def findTreatedVariables(self):
		if self.listOfExperiments is not None and len(self.listOfExperiments) > 0:

			variables = []
			for experiment in self.listOfExperiments:
				variables += experiment.getTreatedVariables()

			if len(variables) > 1:
				variables = list(set(variables))

			var_objects = []
			for variable in variables:
				if variable in self.workingModel.listOfSpecies.names():
					var_objects.append(self.workingModel.listOfVariables.getByName(variable).getSbmlId())

			return var_objects

	def runOptimization(self, nb_procs, timeout=None, maxiter=None):

		self.writeOptimizationFiles(nb_procs)
		return Optimization.runOptimization(self, nb_procs, timeout, maxiter)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.SbmlObject import SbmlObject

from libsignetsim.model.sbml.Constraint import Constraint
from libsignetsim.settings.Settings import Settings


class ListOfConstraints(ListOf, HasIds, SbmlObject):
	""" Class for the listOfConstraints in a sbml model """

	def __init__ (self, model=None):

		self.__model = model
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		SbmlObject.__init__(self, model)


	def readSbml(self, sbml_list_of_constraints,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads a constraints' list from a sbml file """

		for constraint in sbml_list_of_constraints:
			t_constraint = Constraint(self.__model, self.nextId())
			t_constraint.readSbml(constraint, sbml_level, sbml_version)
			ListOf.add(self, t_constraint)

		SbmlObject.readSbml(self, sbml_list_of_constraints, sbml_level, sbml_version)


	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes a constraints' list to a sbml file """

		for constraint in self:
			constraint.writeSbml(sbml_model,sbml_level, sbml_version)

		if len(self) > 0:
			SbmlObject.writeSbml(self, sbml_model.getListOfConstraints(), sbml_level, sbml_version)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made for 'high level' tests, using various components

"""

from libsignetsim import SbmlDocument, SedmlDocument, Settings
from unittest import TestCase
from os.path import join, dirname, isdir
from os import mkdir, getcwd


class TestXMLChanges(TestCase):
	""" Tests high level functions """

	def testAddXML(self):

		testfiles_path = join(dirname(join(getcwd(), __file__)), "files")

		if not isdir(testfiles_path):
			mkdir(testfiles_path)

		# Setting up the model
		sbml_doc = SbmlDocument()
		m = sbml_doc.model
		m.setName("Test")

		s = m.listOfSpecies.new("S", value=5)
		p0 = m.listOfParameters.new("P0")
		sbml_filename = join(Settings.tempDirectory, "testChanges.sbml")
		sbml_doc.writeSbmlToFile(sbml_filename)

		sedml_doc = SedmlDocument()

		simulation = sedml_doc.listOfSimulations.createUniformTimeCourse()
		simulation.setInitialTime(0)
		simulation.setOutputStartTime(0)
		simulation.setOutputEndTime(1)
		simulation.setNumberOfPoints(1)
		simulation.getAlgorithm().setCVODE()

		model = sedml_doc.listOfModels.createModel()
		model.setLanguageSbml()
		model.setSource(sbml_filename)

		addXMLParameter = model.listOfChanges.createAddXML()
		addXMLParameter.setTarget(m.listOfParameters)
		addXMLParameter.setNewXMLFromString("<parameter id=\"test\" name=\"Test\" value=\"2\"/>")

		addXMLInitialAssignment = model.listOfChanges.createAddXML()
		addXMLInitialAssignment.setTarget(m.listOfInitialAssignments)
		addXMLInitialAssignment.setNewXMLFromString(
			"<initialAssignment symbol=\"%s\">" % s.getSbmlId()
			+ "<math xmlns=\"http://www.w3.org/1998/Math/MathML\"><ci>test</ci></math>"
			+ "</initialAssignment>"
		)

		task = sedml_doc.listOfTasks.createTask()
		task.setModel(model)
		task.setSimulation(simulation)

		data_time = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_time.setName("Time")
		var_time = data_time.listOfVariables.createVariable("time")
		var_time.setTask(task)
		var_time.setModel(model)
		var_time.setSymbolTime()
		data_time.getMath().setInternalMathFormula(var_time.getSympySymbol())

		# Let's try to first reproduce the Basal on the figure, aka the fraction of total receptors
		# So we need all vars to compute the fraction
		data_s = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_s.setName("S")

		var_s = data_s.listOfVariables.createVariable("S")
		var_s.setTask(task)
		var_s.setModel(model)
		var_s.setTarget(s)

		data_s.getMath().setInternalMathFormula(var_s.getSympySymbol())

		plot = sedml_doc.listOfOutputs.createPlot2D()
		curve_basal = plot.listOfCurves.createCurve()
		curve_basal.setXData(data_time)
		curve_basal.setYData(data_s)

		model.getSbmlModel()
		sedml_doc.run()
		self.assertEqual(sedml_doc.listOfOutputs[0].listOfCurves[0].getXData(),	[0, 1])
		self.assertEqual(sedml_doc.listOfOutputs[0].listOfCurves[0].getYData(), [2, 2])

		sedml_filename = join(Settings.tempDirectory, "testChanges.sedml")
		sedml_doc.writeSedmlToFile(sedml_filename)

		sedml_doc_2 = SedmlDocument()
		sedml_doc_2.readSedmlFromFile(sedml_filename)
		sedml_doc_2.run()
		self.assertEqual(sedml_doc_2.listOfOutputs[0].listOfCurves[0].getXData(), [0, 1])
		self.assertEqual(sedml_doc_2.listOfOutputs[0].listOfCurves[0].getYData(), [2, 2])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the XPath building and resolution

"""

from libsignetsim import SbmlDocument, KineticLaw

from unittest import TestCase
from os.path import join, dirname


class TestXPaths(TestCase):
	""" Tests high level functions """

	def testResolveXPath(self):

		doc = SbmlDocument()
		model = doc.model
		model.setDefaultUnits()

		c = model.listOfCompartments.new()
		c.setName("Cell")

		s1 = model.listOfSpecies.new()
		s1.setName("Protein")

		p1 = model.listOfParameters.new(value=42)
		p1.setName("Kinetic constant")

		r1 = model.listOfReactions.new()
		r1.setName("Protein degradation")
		r1.listOfReactants.add(s1)
		r1.setKineticLaw(KineticLaw.MASS_ACTION, reversible=False, parameters=[p1])

		e1 = model.listOfEvents.new()
		e1.setName("Reaction stop")
		e1.setTrigger("reaction_0 > 1")
		e1.addEventAssignment(p1, 0)

		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfCompartments/compartment[@name='Cell']"), c)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfCompartments/compartment[@id='compartment_0']"), c)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfCompartments/compartment[@id='compartment_0']/@size"), 1)

		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfSpecies/species[@name='Protein']"), s1)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfSpecies/species[@id='species_0']"), s1)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfSpecies/species[@id='species_0']/@value"), 0)

		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@name='Kinetic constant']"), p1)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='parameter_0']"), p1)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='parameter_0']/@value"), 42)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='parameter_0']/@name"), "Kinetic constant")
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='parameter_0']/@id"), "parameter_0")

		doc.setByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='parameter_0']/@value", 51)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='parameter_0']/@value"), 51)
		doc.setByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='parameter_0']/@name", "Degradation constant")
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='parameter_0']/@name"), "Degradation constant")
		doc.setByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='parameter_0']/@id", "kdeg")
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfParameters/parameter[@id='kdeg']/@id"), "kdeg")

		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfReactions/reaction[@name='Protein degradation']"), r1)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfReactions/reaction[@id='reaction_0']"), r1)

		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfEvents/event[@name='Reaction stop']"), e1)
		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/listOfEvents/event[@id='event_0']"), e1)

		self.assertEqual(doc.getByXPath("/sbml:sbml/sbml:model/descendant::*[@id='species_0']"), s1)
		self.assertEqual(s1.getXPath(), "/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='species_0']")
		self.assertEqual(s1.getXPath("value"), "/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='species_0']/@value")

		self.assertEqual(c.getXPath(), "/sbml:sbml/sbml:model/sbml:listOfCompartments/sbml:compartment[@id='compartment_0']")
		self.assertEqual(r1.getXPath(), "/sbml:sbml/sbml:model/sbml:listOfReactions/sbml:reaction[@id='reaction_0']")
		self.assertEqual(p1.getXPath(), "/sbml:sbml/sbml:model/sbml:listOfParameters/sbml:parameter[@id='kdeg']")
		self.assertEqual(e1.getXPath(), "/sbml:sbml/sbml:model/sbml:listOfEvents/sbml:event[@id='event_0']")

		# Here should we point to the modelDefinition/externalModelDefinition ? Or to the submodels ?
		# Submodels clearly sounds better. I get both should work for the getBy and setBy,
		# but getXPath should probably return the latter

		doc_comp = SbmlDocument()
		doc_comp.enableComp()

		submodel_def = doc_comp.listOfModelDefinitions.new()
		submodel_def.setSbmlId("submodel_def_0")
		submodel_def.setName("Module Definition 0")

		sub_c = submodel_def.listOfCompartments.new()
		sub_c.setName("Cell")

		sub_s1 = submodel_def.listOfSpecies.new()
		sub_s1.setName("Protein")

		sub_p1 = submodel_def.listOfParameters.new()
		sub_p1.setName("Kinetic constant")

		self.assertEqual(doc_comp.getByXPath("/sbml:sbml/sbml:listOfModelDefinitions/modelDefinition[@id='submodel_def_0']/listOfCompartments/compartment[@name='Cell']"), sub_c)
		self.assertEqual(doc_comp.getByXPath("/sbml:sbml/sbml:listOfModelDefinitions/modelDefinition[@id='submodel_def_0']/listOfCompartments/compartment[@id='compartment_0']"), sub_c)
		self.assertEqual(doc_comp.getByXPath("/sbml:sbml/sbml:listOfModelDefinitions/modelDefinition[@id='submodel_def_0']/listOfSpecies/species[@name='Protein']"), sub_s1)
		self.assertEqual(doc_comp.getByXPath("/sbml:sbml/sbml:listOfModelDefinitions/modelDefinition[@id='submodel_def_0']/listOfSpecies/species[@id='species_0']"), sub_s1)
		self.assertEqual(doc_comp.getByXPath("/sbml:sbml/sbml:listOfModelDefinitions/modelDefinition[@id='submodel_def_0']/listOfParameters/parameter[@name='Kinetic constant']"), sub_p1)
		self.assertEqual(doc_comp.getByXPath("/sbml:sbml/sbml:listOfModelDefinitions/modelDefinition[@id='submodel_def_0']/listOfParameters/parameter[@id='parameter_0']"), sub_p1)

		submodel = doc_comp.model.listOfSubmodels.new()
		submodel.setSbmlId("submodel_0")
		submodel.setName("Module 0")
		submodel.setModelRef(submodel_def.getSbmlId())

		self.assertEqual(doc_comp.getByXPath(
			"/sbml:sbml/sbml:model/sbml:listOfSubmodels/submodel[@id='submodel_0']/listOfCompartments/compartment[@name='Cell']"),
						 sub_c
		)
		doc_comp.setByXPath("/sbml:sbml/sbml:model/sbml:listOfSubmodels/submodel[@id='submodel_0']/listOfCompartments/compartment[@name='Cell']/@name", "Cytoplasm")
		self.assertEqual(sub_c.getName(), "Cytoplasm")

		doc_comp_external = SbmlDocument()
		doc_comp_external.readSbmlFromFile(join(dirname(__file__), "files", "comp_model", "modelz9xdww.xml"))
		sos = doc_comp_external.listOfExternalModelDefinitions.getBySbmlId("sos_mod_def").modelDefinition.listOfSpecies.getBySbmlId("sos")
		self.assertEqual(
			doc_comp_external.getByXPath("/sbml:sbml/sbml:listOfExternalModelDefinitions/externalModelDefinition[@id='sos_mod_def']/sbml:sbml/sbml:model/listOfSpecies/species[@id='sos']"),
			sos
		)

		doc_comp_external.setByXPath("/sbml:sbml/sbml:listOfExternalModelDefinitions/externalModelDefinition[@id='sos_mod_def']/sbml:sbml/sbml:model/listOfSpecies/species[@id='sos']/@name", "Son of Sevenless"),
		self.assertEqual(sos.getName(), "Son of Sevenless")

		self.assertEqual(doc_comp_external.getByXPath("/sbml:sbml/sbml:model/sbml:listOfSubmodels/sbml:submodel[@id='sos_mod']/sbml:listOfSpecies/sbml:species[@id='sos']/@name"), "Son of Sevenless")
		doc_comp_external.setByXPath(
			"/sbml:sbml/sbml:model/sbml:listOfSubmodels/sbml:submodel[@id='sos_mod']/sbml:listOfSpecies/sbml:species[@id='sos']/@name", "SOS")
		self.assertEqual(sos.getName(), "SOS")

		self.assertEqual(
			"/sbml:sbml/sbml:listOfExternalModelDefinitions/sbml:externalModelDefinition[@id='sos_mod_def']/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='sos']",
			sos.getXPath()
		)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.cwriter.CModelWriter import CModelWriter
from libsignetsim.model.math.container.ListOfConservationLaws import ListOfConservationLaws
from libsignetsim.model.math.container.ListOfODEs import ListOfODEs
from libsignetsim.model.math.container.ListOfCFEs import ListOfCFEs
from libsignetsim.model.math.container.ListOfDAEs import ListOfDAEs
from libsignetsim.model.math.container.ListOfInitialConditions import ListOfInitialConditions

from libsignetsim.model.math.MathStoichiometryMatrix import MathStoichiometryMatrix
from libsignetsim.model.math.MathConservationMatrix import MathConservationMatrix
from libsignetsim.model.math.MathSlowModel import MathSlowModel
from libsignetsim.model.math.MathAsymmetricModel import MathAsymmetricModel

from libsignetsim.settings.Settings import Settings

from time import time


class MathModel(CModelWriter):
	""" Sbml model class """

	def __init__(self, obj_id=0):
		""" Constructor of model class """

		CModelWriter.__init__(self, obj_id)

		self.listOfODEs = ListOfODEs(self)
		self.listOfCFEs = ListOfCFEs(self)
		self.listOfDAEs = ListOfDAEs(self)

		self.listOfInitialConditions = ListOfInitialConditions(self)

		self.slowModel = None
		self.asymetricModel = MathAsymmetricModel(self)
		self.stoichiometryMatrix = MathStoichiometryMatrix(self)
		self.conservationMatrix = MathConservationMatrix(self)
		self.listOfConservationLaws = ListOfConservationLaws(self)
		self.slowModel = MathSlowModel(self, self.asymetricModel)

		self.nbOdes = None
		self.nbAssignments = None
		self.nbConstants = None
		self.nbAlgebraics = None

		self.variablesOdes = None
		self.variablesAssignment = None
		self.variablesConstant = None
		self.variablesAlgebraic = None
		self.__upToDate = False


	def isUpToDate(self):
		return self.__upToDate

	def setUpToDate(self, value=True):
		self.__upToDate = value

	def getMathModel(self):

		if self.slowModel.isUpToDate():
			return self.slowModel

		elif self.asymetricModel.isUpToDate():
			return self.asymetricModel
		else:
			return self

	def buildModel(self, vars_to_keep=[], reduce=Settings.reduceByDefault, tmin=0):

		if not reduce:
			self.slowModel.setUpToDate(False)
			self.asymetricModel.setUpToDate(False)

		self.listOfCFEs.build()
		self.listOfODEs.build()
		self.listOfDAEs.build()

		t0 = time()

		self.listOfInitialConditions.solve(tmin)

		if Settings.verboseTiming >= 1:
			print("> Initial condition solved in %.2gs" % (time()-t0))

		if len(self.listOfDAEs) > 0:
			self.listOfDAEs.solveInitialConditions(tmin)
			self.listOfDAEs.solveDAEs()

		if reduce:
			self.buildReducedModel(vars_to_keep=vars_to_keep)

		if len(self.listOfEvents) == 0 and self.listOfReactions.hasFastReaction():
			self.buildSlowModel()

		self.setUpToDate()

	def buildConservationLaws(self):

		t0 = time()
		self.stoichiometryMatrix.build()
		if Settings.verboseTiming >= 2:
			print("> stoichiometry matrix built in %.2gs" % (time() -t0))

		t0 = time()
		self.listOfConservationLaws.build()
		if Settings.verboseTiming >= 2:
			print("> conservation laws built in %.2gs" % (time() - t0))

	def buildReducedModel(self, vars_to_keep=[]):

		t0 = time()
		self.stoichiometryMatrix.build()
		self.listOfConservationLaws.build()
		self.asymetricModel.build(treated_variables=vars_to_keep)
		if Settings.verboseTiming >= 1:
			print("> model reduced in %.2gs" % (time() - t0))

	def buildSlowModel(self):

		vars_to_keep = (
			[var.getSymbolStr() for var in self.listOfVariables.getFastVariables()]
			+ [var.getSymbolStr() for var in self.listOfVariables.getSlowVariables()]
		)

		self.buildReducedModel(vars_to_keep=vars_to_keep)
		self.slowModel.build()

	def prettyPrint(self):

		print("\n> Full system : ")

		print(self.listOfCFEs)
		print(self.listOfDAEs)
		print(self.listOfODEs)
		print(self.listOfConservationLaws)

		print("-----------------------------")

	def pprint(self):
		self.listOfCFEs.pprint()
		self.listOfDAEs.pprint()
		self.listOfODEs.pprint()
		self.listOfInitialConditions.pprint()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.ModelException import InvalidXPath

from libsignetsim.model.sbml.ExternalModelDefinition import ExternalModelDefinition
from libsignetsim.settings.Settings import Settings
from re import match

class ListOfExternalModelDefinitions(ListOf, HasIds, HasParentObj):
	""" Class for the listOfExternalModelDefinition in a sbml model """

	def __init__(self, model, parent_obj):

		self.__model = model
		HasParentObj.__init__(self, parent_obj)
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		# SbmlObject.__init__(self, model)


	def readSbml(self, sbml_external_models,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads external model definitions' list from a sbml file """

		for model in sbml_external_models:
			t_model = ExternalModelDefinition(self.__model, self.nextId(), self)
			t_model.readSbml(model, sbml_level, sbml_version)
			ListOf.add(self, t_model)

		# SbmlObject.writeSbml(self, sbml_external_models, sbml_level, sbml_version)

	def writeSbml(self, sbml_document,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes external model definitions' list to a sbml file """

		for model in self:
			sbml_emd = sbml_document.createExternalModelDefinition()
			model.writeSbml(sbml_emd, sbml_level, sbml_version)

		# SbmlObject.writeSbml(self, sbml_document, sbml_level, sbml_version)

	def new(self):
		""" Creates a new external model definition """

		t_model = ExternalModelDefinition(self.__model, self.nextId(), self)
		ListOf.add(self, t_model)
		return t_model

	def getListOfModelDefinitions(self):
		res = []
		for external_model in self:
			res.append(external_model)
		return res

	def resolveXPath(self, selector):

		if selector.startswith("externalModelDefinition") or selector.startswith("sbml:externalModelDefinition"):

			res_match = match(r'(.+)\[@(.+)=\'(.+)\'\]', selector)

			if res_match is not None:
				tokens = res_match.groups()
				if len(tokens) == 3:
					if tokens[1] == "id":
						return self.getBySbmlId(tokens[2])
					elif tokens[1] == "name":
						return self.getByName(tokens[2])

		# If not returned yet
		raise InvalidXPath(selector)

	def getByXPath(self, xpath):
		return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])

	def setByXPath(self, xpath, value):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], value)

	def getXPath(self):
		return "/".join([self.getParentObj().getXPath(), "sbml:listOfExternalModelDefinitions"])




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.model.math.MathException import MathException
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs
from libsignetsim.model.math.sympy_shortcuts import SympySymbol, SympyEqual, SympyInteger, SympyFloat
from libsignetsim.settings.Settings import Settings

from sympy import pretty
from time import time


class ListOfInitialConditions(dict):
	""" Sbml model class """

	def __init__(self, model):
		""" Constructor of model class """

		dict.__init__(self)
		self.__model = model

	def clear(self):
		dict.clear(self)

	def __str__(self):
		res = ""
		for var, value in dict.items(self):
			res += "%s : %s\n" % (var.symbol.getInternalMathFormula(), value.getDeveloppedInternalMathFormula())
		return res

	def pprint(self):

		for var, value in dict.items(self):
			var0 = SympySymbol("(" + str(var) + ")_0")
			print(pretty(SympyEqual(var0, value.getDeveloppedInternalMathFormula())))
			print("\n")


	def solve(self, tmin):
		""" Initial conditions are a mess between values, initial assignments,
			and assignment rules. We actually need to solve them to make sure
			all dependencies are respected

			Unfortunately, this can be quite costly for large system
			So we'll try to just solve the minimum for C simulations

			"""

		DEBUG = False

		t0 = time()

		if tmin == 0:
			init_cond = {SympySymbol("_time_"): SympyInteger(tmin)}
		else:
			init_cond = {SympySymbol("_time_"): SympyFloat(tmin)}

		for init_ass in self.__model.listOfInitialAssignments:
			if init_ass.isValid():
				t_var = init_ass.getVariable().symbol.getSymbol()
				t_value = init_ass.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula()
				init_cond.update({t_var: t_value})

		if DEBUG:
			print(init_cond)

		for rule in self.__model.listOfRules:
			if rule.isAssignment() and rule.isValid():
				t_var = rule.getVariable().symbol.getSymbol()
				if t_var not in list(init_cond.keys()):
					t_value = rule.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula()
					init_cond.update({t_var: t_value})

		if DEBUG:
			print(init_cond)

		for var in self.__model.listOfVariables:
			t_var = var.symbol.getSymbol()
			if t_var not in list(init_cond.keys()):
				t_value = var.value.getDeveloppedInternalMathFormula()
				if t_value is not None:
					init_cond.update({t_var: t_value})
				elif not var.isAlgebraic():
					init_cond.update({t_var: SympyFloat(0.0)})

		if DEBUG:
			print(init_cond)

		crossDependencies = True
		passes = 1
		while crossDependencies:

			if DEBUG:
				print("PASS : %d" % passes)

			crossDependencies = False

			for t_var in list(init_cond.keys()):
				t_def = init_cond[t_var]
				if len(t_def.atoms(SympySymbol).intersection(set(init_cond.keys()))) > 0:
					crossDependencies = True

					if DEBUG:
						print("\n> " + str(t_var) + " : " + str(t_def))

					for match in t_def.atoms(SympySymbol).intersection(set(init_cond.keys())):
						if match == t_var:
							raise MathException("Initial values : self dependency is bad")
						if DEBUG:
							print(">> " + str(match) + " : " + str(init_cond[match]))

						t_def = unevaluatedSubs(t_def, {match: init_cond[match]})
						init_cond.update({t_var: t_def})

					if DEBUG:
						if len(t_def.atoms(SympySymbol).intersection(set(init_cond.keys()))) == 0:
							print("> " + str(t_var) + " : " + str(t_def) + " [OK]")
						else:
							print("> " + str(t_var) + " : " + str(t_def) + " [ERR]")
			passes += 1
			if passes >= 100:
				raise MathException("Initial values : Probable circular dependencies")

			if DEBUG:
				print("")

		if DEBUG:
			print(list(init_cond.keys()))
			print(self.__model.listOfVariables.symbols())

		# self.listOfInitialConditions = {}
		dict.clear(self)
		for var, value in list(init_cond.items()):
			if var != SympySymbol("_time_"):
				t_var = self.__model.listOfVariables.getBySymbol(var)
				if t_var is not None:
					t_value = MathFormula(self.__model)
					t_value.setInternalMathFormula(value.doit())
					dict.update(self, {t_var.symbol.getSymbol(): t_value})

		if DEBUG:
			print("> Final listOfInitialConditions")
			print(dict.keys(self))

		for var in self.__model.listOfVariables:
			if not var.symbol.getSymbol() in dict.keys(self) and not var.isAlgebraic():
				print("Lacks an initial condition : %s" % var.getSbmlId())

		if Settings.verbose >= 1:
			print("> Finished calculating initial conditions (%.2gs)" % (time()-t0))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""



from libsignetsim.uris.URI import URI
from libsignetsim.settings.Settings import Settings
from libsbml import (
	MODEL_QUALIFIER, BIOLOGICAL_QUALIFIER, BQB_ENCODES, BQB_HAS_PART, BQB_HAS_PROPERTY, BQB_HAS_TAXON, BQB_HAS_VERSION,
	BQB_IS, BQB_IS_DESCRIBED_BY, BQB_IS_ENCODED_BY, BQB_IS_HOMOLOG_TO, BQB_IS_PART_OF, BQB_IS_PROPERTY_OF,
	BQB_IS_VERSION_OF, BQB_OCCURS_IN, BQB_UNKNOWN, BQM_HAS_INSTANCE, BQM_IS, BQM_IS_DERIVED_FROM, BQM_IS_DESCRIBED_BY,
	BQM_IS_INSTANCE_OF, BQM_UNKNOWN
)

class CVTerm(object):

	biological_qualifiers = {
		BQB_ENCODES: "encodes",
		BQB_HAS_PART: "hasPart",
		BQB_HAS_PROPERTY: "hasProperty",
		BQB_HAS_TAXON: "hasTaxon",
		BQB_HAS_VERSION: "hasVersion",
		BQB_IS: "is",
		BQB_IS_DESCRIBED_BY: "isDescribedBy",
		BQB_IS_ENCODED_BY: "isEncodedBy",
		BQB_IS_HOMOLOG_TO: "isHomologTo",
		BQB_IS_PART_OF: "isPartOf",
		BQB_IS_PROPERTY_OF: "isPropertyOf",
		BQB_IS_VERSION_OF: "isVersionOf",
		BQB_OCCURS_IN: "occursIn",
		BQB_UNKNOWN: "unknown",
	}
	model_qualifiers = {
		BQM_HAS_INSTANCE: "hasInstance",
		BQM_IS: "is",
		BQM_IS_DERIVED_FROM: "isDerivedFrom",
		BQM_IS_DESCRIBED_BY: "isDescribedBy",
		BQM_IS_INSTANCE_OF: "isInstanceOf",
		BQM_UNKNOWN: "unknown"
	}

	def __init__(self, model):

		self.__model = model
		self.__qualifierType = None
		self.__qualifier = None
		self.__resources = []


	def readSbml(self, cv_term,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		self.__qualifierType = cv_term.getQualifierType()

		if self.__qualifierType == MODEL_QUALIFIER:
			self.__qualifier = cv_term.getModelQualifierType()
		elif self.__qualifierType == BIOLOGICAL_QUALIFIER:
			self.__qualifier = cv_term.getBiologicalQualifierType()

		for i_res in range(cv_term.getResources().getNumAttributes()):
			t_uri = URI()
			t_uri.readURI(cv_term.getResourceURI(i_res))
			self.__resources.append(t_uri)

	def writeSbml(self, cv_term,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):


		if self.__qualifierType == MODEL_QUALIFIER:

			cv_term.setQualifierType(self.__qualifierType)
			cv_term.setModelQualifierType(self.__qualifier)
			for uri in self.__resources:
				cv_term.addResource(uri.writeURI())

		elif self.__qualifierType == BIOLOGICAL_QUALIFIER:
			cv_term.setQualifierType(self.__qualifierType)
			cv_term.setBiologicalQualifierType(self.__qualifier)
			for uri in self.__resources:
				cv_term.addResource(str(uri.writeURI()))

	def __str__(self):

		res = ""
		if self.__qualifierType == MODEL_QUALIFIER:
			res += ">> Model qualifier\n"
			for uri in self.__resources:
				res += ">>> %s : %s\n" % (self.model_qualifiers[self.__qualifier], self(uri))

		elif self.__qualifierType == BIOLOGICAL_QUALIFIER:
			res += ">> Biological\n"
			for uri in self.__resources:
				res += ">>> %s : %s\n" % (self.biological_qualifiers[self.__qualifier], self(uri))

		return res

	def getURIs(self):
		return self.__resources

	def addURI(self, uri):
		self.__resources.append(uri)

	def isHasPart(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_HAS_PART

	def isHasProperty(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_HAS_PROPERTY

	def isHasTaxon(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_HAS_TAXON

	def isHasVersion(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_HAS_VERSION

	def isIs(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_IS

	def isIsDescribedBy(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_IS_DESCRIBED_BY

	def setIsDescribedBy(self):

		self.__qualifierType = BIOLOGICAL_QUALIFIER
		self.__qualifier = BQB_IS_DESCRIBED_BY

	def isIsEncodedBy(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_IS_ENCODED_BY

	def isIsHomologTo(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_IS_HOMOLOG_TO

	def isIsPartOf(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_IS_PART_OF

	def isIsPropertyOf(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_IS_PROPERTY_OF

	def isIsVersionOf(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_IS_VERSION_OF

	def isOccursIn(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_OCCURS_IN

	def isUnknown(self):
		return self.__qualifierType == BIOLOGICAL_QUALIFIER and list(self.biological_qualifiers.keys())[self.__qualifier] == BQB_UNKNOWN

	def isModelHasInstance(self):
		return self.__qualifierType == MODEL_QUALIFIER and list(self.model_qualifiers.keys())[self.__qualifier] == BQM_HAS_INSTANCE

	def isModelIs(self):
		return self.__qualifierType == MODEL_QUALIFIER and list(self.model_qualifiers.keys())[self.__qualifier] == BQM_IS

	def isModelIsDerivedFrom(self):
		return self.__qualifierType == MODEL_QUALIFIER and list(self.model_qualifiers.keys())[self.__qualifier] == BQM_IS_DERIVED_FROM

	def isModelIsDescribedBy(self):
		return self.__qualifierType == MODEL_QUALIFIER and list(self.model_qualifiers.keys())[self.__qualifier] == BQM_IS_DESCRIBED_BY

	def isModelIsInstanceOf(self):
		return self.__qualifierType == MODEL_QUALIFIER and list(self.model_qualifiers.keys())[self.__qualifier] == BQM_IS_INSTANCE_OF

	def isModelUnknown(self):
		return self.__qualifierType == MODEL_QUALIFIER and list(self.model_qualifiers.keys())[self.__qualifier] == BQM_UNKNOWN


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.Parameter import Parameter
from libsignetsim.settings.Settings import Settings


class ListOfParameters(ListOf):

	def __init__(self, document, parent):

		ListOf.__init__(self, document)

		self.__document = document
		self.__parent = parent
		self.__parameterCounter = 0

	def new(self, parameter_id=None):

		if parameter_id is None:
			parameter_id = "%s_parameter_%d" % self.__parameterCounter

		t_parameter = Parameter(self.__document)
		t_parameter.setId(parameter_id)
		ListOf.append(self, t_parameter)
		self.__parameterCounter += 1
		return t_parameter
	def createParameter(self, parameter_id=None):
		return self.new(parameter_id)

	def readSedml(self, list_of_parameters, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_parameters, level, version)

		for t_parameter in list_of_parameters:
			parameter = Parameter(self.__document)
			parameter.readSedml(t_parameter, level, version)
			ListOf.append(self, parameter)
			self.__parameterCounter += 1

	def writeSedml(self, list_of_parameters, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_parameters, level, version)

		for t_parameter in self:
			parameter = list_of_parameters.createParameter()
			t_parameter.writeSedml(parameter, level, version)

	def getSubs(self):

		data = {}
		for parameter in self:
			data.update(parameter.getSub())

		return data


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.EventAssignedVariable import EventAssignedVariable
from libsignetsim.model.sbml.InitiallyAssignedVariable import InitiallyAssignedVariable
from libsignetsim.model.sbml.RuledVariable import RuledVariable
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.Variable import Variable
from libsignetsim.model.math.sympy_shortcuts import *
from libsignetsim.settings.Settings import Settings

from libsbml import AST_REAL, AST_INTEGER, AST_RATIONAL, formulaToString
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs
from sympy import Symbol, sympify, srepr, simplify, expand, factor, nsimplify, Integer

class SpeciesReference(SbmlObject, Variable, InitiallyAssignedVariable,
						RuledVariable, EventAssignedVariable):

	def __init__ (self, model, objId, is_modifier=False):

		SbmlObject.__init__(self, model)
		self.__model = model
		self.objId = objId
		self.__isModifier = is_modifier
		self.__species = None
		self.stoichiometry = MathFormula(model)
		self.constant = True # True by default sounds fair
		self.__hasId = False


	def load(self, species, stoichiometry=None):

		self.__species = species.getSbmlId()

		if stoichiometry is None:
			self.stoichiometry.setValueMathFormula(1)

		else:
			self.stoichiometry.setValueMathFormula(stoichiometry)


	def copy(self, obj, sids_subs={}, symbols_subs={}):

		SbmlObject.copy(self, obj)
		if obj.hasId():

			Variable.__init__(self, self.__model, Variable.STOICHIOMERY)
			InitiallyAssignedVariable.__init__(self, self.__model)
			RuledVariable.__init__(self, self.__model)
			EventAssignedVariable.__init__(self, self.__model)

			Variable.copy(self, obj, sids_subs=sids_subs, symbols_subs=symbols_subs)
			self.constant = obj.constant

		t_stoichiometry = unevaluatedSubs(obj.stoichiometry.getInternalMathFormula(), symbols_subs)
		self.stoichiometry.setInternalMathFormula(t_stoichiometry)

		if obj.getSpecies().getSbmlId() in list(sids_subs.keys()):
			self.__species = sids_subs[obj.getSpecies().getSbmlId()]
		else:
			self.__species = obj.getSpecies().getSbmlId()

		self.__isModifier = obj.isModifier()


	def readSbml(self, sbmlSpeciesReference, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Reads a species reference from a sbml file """

		SbmlObject.readSbml(self, sbmlSpeciesReference, sbml_level, sbml_version)
		if sbml_level >= 3 and sbmlSpeciesReference.isSetId():
			Variable.__init__(self, self.__model, Variable.STOICHIOMERY)
			InitiallyAssignedVariable.__init__(self, self.__model)
			RuledVariable.__init__(self, self.__model)
			EventAssignedVariable.__init__(self, self.__model)
			Variable.readSbml(self, sbmlSpeciesReference, sbml_level, sbml_version)
			self.stoichiometry = self.symbol
			self.__hasId = True
			self.__isModifier = sbmlSpeciesReference.isModifier()
		else:


			if sbmlSpeciesReference.isModifier():
				self.__isModifier = True
				self.stoichiometry.setValueMathFormula(1)

			else:
				if sbmlSpeciesReference.isSetStoichiometry():

					if sbml_level == 1:
						self.stoichiometry.setInternalMathFormula(
							SympyMul(
								SympyFloat(float(sbmlSpeciesReference.getStoichiometry())),
								SympyPow(SympyFloat(float(sbmlSpeciesReference.getDenominator())),
											SympyInteger(-1))))

					else:
						self.stoichiometry.readSbml(
							sbmlSpeciesReference.getStoichiometry())

				elif sbmlSpeciesReference.isSetStoichiometryMath():
					self.stoichiometry.readSbml(
						sbmlSpeciesReference.getStoichiometryMath().getMath(),
						sbml_level, sbml_version)


				else:
					self.stoichiometry.setValueMathFormula(1)


		if sbml_level >= 2:
			self.__species = sbmlSpeciesReference.getSpecies()
		else:
			self.__species = sbmlSpeciesReference.getSpecies()


	def readSbmlVariable(self, sbml_species_reference, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		self.symbol.readSbml(sbml_species_reference.getId(), sbml_level, sbml_version)
		if sbml_level == 3 and not sbml_species_reference.isModifier():
			self.constant = sbml_species_reference.getConstant()
		else:
			self.constant = True

		if not sbml_species_reference.isModifier() and sbml_species_reference.isSetStoichiometry():
			self.isInitialized = True
			self.value.readSbml(sbml_species_reference.getStoichiometry(), sbml_level, sbml_version)
		else:
			self.isInitialized = True
			self.value.setInternalMathFormula(Integer(1))

	def writeSbml(self, sbml_speciesReference, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes a species reference to  a sbml file """

		sbml_speciesReference.setSpecies(self.__species)
		SbmlObject.writeSbml(self, sbml_speciesReference, sbml_level, sbml_version)

		if sbml_level == 3 and self.__hasId:
			Variable.writeSbml(self, sbml_speciesReference, sbml_level, sbml_version)


		elif not sbml_speciesReference.isModifier():
			if sbml_level == 1:
				t_stoichiometry = nsimplify(self.stoichiometry.getInternalMathFormula())
				if t_stoichiometry.is_Rational and t_stoichiometry.q != 1:

					sbml_speciesReference.setStoichiometry(int(t_stoichiometry.p))
					sbml_speciesReference.setDenominator(int(t_stoichiometry.q))

				elif t_stoichiometry != MathFormula.ONE:
					# print srepr(t_stoichiometry)
					# t_stoichiometry = self.stoichiometry.getSbmlMathFormula(sbml_level, sbml_version)
					sbml_speciesReference.setStoichiometry(int(t_stoichiometry))

			else:

				t_stoichiometry = self.stoichiometry.getSbmlMathFormula(sbml_level, sbml_version)

				if t_stoichiometry.getType() == AST_REAL:
					if t_stoichiometry.getReal() != 1.0 or sbml_level == 3:
						sbml_speciesReference.setStoichiometry(t_stoichiometry.getReal())

				elif t_stoichiometry.getType() == AST_INTEGER:
					if t_stoichiometry.getInteger() != 1 or sbml_level == 3:
						sbml_speciesReference.setStoichiometry(t_stoichiometry.getInteger())

				elif t_stoichiometry.getType() == AST_RATIONAL:
					sbml_speciesReference.setStoichiometry(t_stoichiometry.getNumerator())
					sbml_speciesReference.setDenominator(t_stoichiometry.getDenominator())

				else:
					if sbml_level == 3:
						if self.__model.listOfVariables.containsSymbol(self.stoichiometry.getInternalMathFormula()):
							sbml_speciesReference.setId(t_stoichiometry.getName())

						else:
							sbml_speciesReference.setStoichiometry(t_stoichiometry)
					else:
						sbml_stoichiometry_math = sbml_speciesReference.createStoichiometryMath()
						sbml_stoichiometry_math.setMath(t_stoichiometry)

			if sbml_level >= 3:
				sbml_speciesReference.setConstant(self.constant)

	def writeSbmlVariable(self, sbml_species_reference, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		if not self.isModifier():

			sbml_species_reference.setConstant(self.constant)

			if self.isInitialized:
				sbml_species_reference.setStoichiometry(self.getValue())


	def setSpecies(self, species):
		self.__species = species.getSbmlId()

	def getSpecies(self):
		if self.__species is not None:
			return self.__model.listOfSpecies.getBySbmlId(self.__species)


	def hasId(self):
		return self.__hasId

	def isModifier(self):
		return self.__isModifier

	def setStoichiometry(self, stoichiometry):

		if isinstance(stoichiometry, int) or isinstance(stoichiometry, float):
			self.stoichiometry.setValueMathFormula(stoichiometry)

		# elif isinstance(stoichiometry, float):
		# 	self.stoichiometry = setPrettyPrintMathFormula(stoichiometry)

	def getStoichiometry(self):
		return self.stoichiometry.getPrettyPrintMathFormula()

	def getStoichiometryMath(self):
		return self.stoichiometry

	def isVariableStoichiometry(self):

		if self.__hasId and not self.constant:
			return True
		elif self.stoichiometry.getInternalMathFormula().func == SympySymbol:
			t_var = self.__model.listOfVariables.getBySymbol(self.stoichiometry.getInternalMathFormula())
			return t_var is not None and not t_var.isConstant()
		else:
			return False
		
	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		if self.__species == old_sbml_id:
			self.__species = new_sbml_id


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasMetaIds import HasMetaIds


class ListOfSbmlObjects(ListOf, HasMetaIds):
	""" Class for the listOfModelDefinition in a sbml model """

	def __init__ (self, model=None):

		self.__model = model
		ListOf.__init__(self, model)
		HasMetaIds.__init__(self, model)
		self.isListOfSbmlObjects = True
		self.currentObjId = -1

	def nextMetaId(self):
		self.currentObjId += 1
		return "_meta_id_%d_" % self.currentObjId

	def addSbmlObject(self, sbml_object, prefix=""):

		if sbml_object.getMetaId() is None:
			sbml_object.setMetaId(self.nextMetaId())

		if self.containsMetaId(sbml_object.getMetaId()):

			t_meta_id = sbml_object.getMetaId()
			while self.containsMetaId(t_meta_id):
				t_meta_id = self.nextMetaId()

			sbml_object.rawSetMetaId(t_meta_id)

		self.append(sbml_object)

		return sbml_object


	def getListOfReplacedElements(self):

		res = []
		for obj in self:
			if obj.hasReplacedElements():
				res += obj.getReplacedElements()

		return res

	def getListOfReplacedBy(self):

		res = []
		for obj in self:
			if obj.isReplaced():
				res.append(obj.getReplacedBy())

		return res

	def getListOfSubstitutions(self):
		return self.getListOfReplacedElements() + self.getListOfReplacedBy()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.ListOfMathVariables import ListOfMathVariables
from libsignetsim.model.ListOfSbmlVariables import ListOfSbmlVariables
from libsignetsim.model.math.sympy_shortcuts import SympySymbol

class ListOfVariables(ListOfMathVariables, ListOfSbmlVariables, list):
	""" Parent class for all the ListOf containers in a sbml model """

	def __init__(self, model):

		self.__model = model
		ListOfMathVariables.__init__(self, model)
		ListOfSbmlVariables.__init__(self, model)
		list.__init__(self)

	def addVariable(self, variable, string=None):

		t_sbmlId = ListOfSbmlVariables.newSbmlId(self, variable, string)
		list.append(self, variable)
		return t_sbmlId

	def removeVariable(self, variable):
		list.remove(self, variable)

	# Symbols
	def symbols(self):
		""" Return a set of symbols of the sbml variables """
		return [obj.symbol.getInternalMathFormula() for obj in self]

	def containsSymbol(self, symbol):
		""" Returns if a symbol is in the list of sbml variables"""
		for var in self:
			if var.symbol.getSymbol() == symbol:
				return True
		return False

	def getBySymbol(self, symbol):
		""" Get a sbml variable by his symbol"""
		for var in self:
			if var.symbol.getSymbol() == symbol:
				return var

	def getBySymbolStr(self, symbol_str):
		""" Get a sbml variable by his symbol string"""
		for var in self:
			if var.getSymbolStr() == symbol_str:
				return var

	# Renaming variable
	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		old_symbol = SympySymbol(old_sbml_id)
		if old_symbol in self.symbols():
			t_var = self.getBySymbol(old_symbol)
			t_var.renameSymbol(old_sbml_id, new_sbml_id)

		for var in self:
			var.renameSbmlIdInValue(old_sbml_id, new_sbml_id)


	def clear(self):
		list.__init__(self)

	def getFastVariables(self):
		return [species for species in self.__model.listOfSpecies if species.isOnlyInFastReactions()]

	def getMixedVariables(self):
		return [species for species in self.__model.listOfSpecies if species.isInFastReactions()]

	def getSlowVariables(self):
		return [species for species in self.__model.listOfSpecies if not species.isInFastReactions()]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from PyDSTool import args, ContClass, PyDSTool_ExistError
from .PyDSToolModel import PyDSToolModel
from libsignetsim.model.math.sympy_shortcuts import SympySymbol
from libsignetsim.settings.Settings import Settings
from os.path import isfile, join, isdir
from os import remove, getcwd
from threading import Thread
from random import choice
from string import ascii_uppercase, ascii_lowercase, digits, punctuation
from shutil import rmtree
from time import time


class EquilibriumPointCurve(object):

	NOT_STARTED = 10
	STARTED 	= 11
	SUCCESS		= 12
	FAILED		= 13

	MAIN_CURVE = 'EQ1'
	LIMIT_CYCLE_CURVE = 'LC1'

	def __init__(self, model):

		self.model = model
		self.system = PyDSToolModel(model)
		self.continuation = None
		self.parameter = None
		self.continuationParameters = None
		self.fromValue = None
		self.toValue = None
		self.ds = 0.1
		self.maxSteps = 1000
		self.verbosity = 0
		self.status = self.NOT_STARTED
		self.elapsedTime = 0
		self.x = []
		self.ys = {}
		self.points = {}
		self.continuationId = ''.join(choice(ascii_uppercase + ascii_lowercase + digits) for _ in range(6))

	def setParameter(self, parameter):
		self.parameter = parameter

	def setRange(self, from_value, to_value):
		self.fromValue = from_value
		self.toValue = to_value

	def setDs(self, ds):
		self.ds = ds

	def setMaxSteps(self, max_steps):
		self.maxSteps = max_steps

	def setVerbosity(self, verbosity):
		self.verbosity = verbosity

	def build(self, vars_to_keep=[]):

		self.system.build(
			self.parameter,
			self.fromValue, vars_to_keep=[self.parameter.getSymbolStr()] + vars_to_keep
		)

		self.buildCont()

	def run_async(self, callback_function_success, callback_function_error):
		self.executeContThread(callback_function_success, callback_function_error)

	def run(self):
		self.executeCont()

	def buildCont(self):

		self.continuation = ContClass(self.system.getSystem())
		self.continuation.setTempDirectory(join(Settings.tempDirectory, "continuation_%s" % self.continuationId))
		self.continuationParameters = args(name=self.MAIN_CURVE, type='EP-C')
		self.continuationParameters.freepars = [self.parameter.getSymbolStr()]
		self.continuationParameters.StepSize = abs(self.ds)
		self.continuationParameters.MaxNumPoints = self.maxSteps
		self.continuationParameters.MaxStepSize = self.ds*100
		self.continuationParameters.MinStepSize = self.ds/100
		self.continuationParameters.LocBifPoints = 'All'
		self.continuationParameters.verbosity = self.verbosity
		self.continuationParameters.SaveEigen = True

		self.continuation.newCurve(self.continuationParameters)

	def executeCont(self, callback_function_success=None, callback_function_error=None):

		self.status = self.STARTED
		startTime = time()
		try:
			if self.ds > 0:
					self.continuation[self.MAIN_CURVE].forward()
			else:
				self.continuation[self.MAIN_CURVE].backward()

			if self.hasHopfBifurcations():
				self.findLimitCycleCurves()

			self.status = self.SUCCESS

			self.buildCurves()
			self.buildPoints()
			self.calculateForExactVariables()
			self.cleanupFiles()
			self.elapsedTime = time() - startTime

			if callback_function_success is not None:
				callback_function_success(self)

		except Exception as e:
			self.status = self.FAILED
			self.elapsedTime = time() - startTime
			if callback_function_error is not None:
				callback_function_error(error=str(e))


	def cleanupFiles(self):

		if isdir(join(Settings.tempDirectory, "continuation_%s" % self.continuationId)):
			rmtree(join(Settings.tempDirectory, "continuation_%s" % self.continuationId))

	def calculateForExactVariables(self):

		for var in self.model.getMathModel().variablesAssignment:
			if not self.model.listOfVariables.getBySymbolStr(var.getSymbolStr()).isReaction():

				cfe = self.model.getMathModel().listOfCFEs.getByVariable(var)
				math_formula = cfe.getDefinition().getDeveloppedInternalMathFormula()
				vars_formula = list(math_formula.atoms(SympySymbol))

				subs_constant = {}

				for constant in self.model.getMathModel().variablesConstant:
					if constant.symbol.getSymbol() in vars_formula and constant.symbol.getSymbol() != self.parameter.symbol.getSymbol():
						init_cond = self.model.getMathModel().listOfInitialConditions[constant.symbol.getSymbol()]
						subs_constant.update({constant.symbol.getSymbol(): init_cond.getInternalMathFormula()})

				math_formula_2 = math_formula.subs(subs_constant)
				vars_formula_2 = math_formula_2.atoms(SympySymbol).difference({self.parameter.symbol.getSymbol()})

				res = []
				for i, x_i in enumerate(self.x):
					subs_variables = {}
					for var_formula in vars_formula_2:
						variable = self.model.getMathModel().listOfVariables.getBySymbol(var_formula).getSymbolStr()
						subs_variables.update({var_formula: self.ys[variable][i]})

					subs_variables.update({self.parameter.symbol.getSymbol(): x_i})

					res.append(math_formula_2.subs(subs_variables))
				self.ys.update({var.getSymbolStr(): res})

				res = []
				for i, (type, x, y) in enumerate(self.points[list(self.points.keys())[0]]):

					subs_variables = {}
					for var_formula in vars_formula_2:
						variable = self.model.getMathModel().listOfVariables.getBySymbol(var_formula).getSymbolStr()
						subs_variables.update({var_formula: self.points[variable][i][2]})

					subs_variables.update({self.parameter.symbol.getSymbol(): x})
					res.append((type, x, math_formula_2.subs(subs_variables)))

				self.points.update({var.getSymbolStr(): res})

	def executeContThread(self, callback_function_success, callback_function_error):

		t = Thread(group=None, target=self.executeCont, args=(callback_function_success, callback_function_error))
		t.setDaemon(True)
		t.start()

	def getCurve(self, variable):

		if self.status == self.SUCCESS:

			x, ys = self.getCurves()
			return x, ys[variable.getSymbolStr()]

	def buildCurves(self):

		if self.status == self.SUCCESS:

			len_curve = len(self.continuation[self.MAIN_CURVE].curve[:, 1]) - 1
			nb_variables = len(self.continuation[self.MAIN_CURVE].varslist)
			self.x = self.continuation[self.MAIN_CURVE].curve[0:len_curve, nb_variables]
			self.x = [float(x_i) for x_i in self.x if self.fromValue <= x_i <= self.toValue]
			self.ys = {}
			for i, var in enumerate(self.continuation[self.MAIN_CURVE].varslist):
				y = self.continuation[self.MAIN_CURVE].curve[0:len_curve, i]
				y = [float(y_i) for x_i, y_i in zip(self.x, y) if self.fromValue <= x_i <= self.toValue]
				self.ys.update({var: y})

	def getCurves(self):

		if self.status == self.SUCCESS:

			return self.x, self.ys

	def getStability(self, x):

		if self.status == self.SUCCESS:

			stability = []
			for i in range(len(self.continuation[self.MAIN_CURVE].sol.labels)):
				label = self.continuation[self.MAIN_CURVE].sol.labels[i]
				if 'EP' in list(label.keys()) and i < len(x) and self.fromValue <= x[i] <= self.toValue:
						stability.append(label['EP']['stab'])

			return stability


	def getStabilitySlicedCurves(self):

		if self.status == self.SUCCESS:

			x, ys = self.getCurves()
			stability = self.getStability(x)

			split = []
			t_res = []
			for i, x_i in enumerate(stability):
				if i == 0:
					t_res.append(i)
				else:
					if x_i == stability[i - 1]:
						t_res.append(i)
					else:
						split.append(t_res)
						t_res = [i]

			split.append(t_res)

			split_stability = []
			split_x = []
			split_ys = {var: [] for var in list(ys.keys())}

			for inds in split:
				split_stability.append(stability[inds[0]])
				split_x.append([float(x[i]) for i in inds])

				for var, y in list(ys.items()):
					split_ys[var].append([float(y[i]) for i in inds])

			return split_x, split_ys, split_stability


	def getLimitCycleCurves(self):

		x = []
		curves = {}

		if self.status == self.SUCCESS and self.LIMIT_CYCLE_CURVE in list(self.continuation.curves.keys()):

			len_curve = len(self.continuation[self.LIMIT_CYCLE_CURVE].curve[:, 1]) - 1
			nb_variables = len(self.continuation[self.LIMIT_CYCLE_CURVE].varslist)
			x = self.continuation[self.LIMIT_CYCLE_CURVE].curve[0:len_curve - 1, nb_variables * 2]
			x = [x_i for x_i in x if self.fromValue <= x_i <= self.toValue]

			for i, var in enumerate(self.continuation[self.LIMIT_CYCLE_CURVE].varslist):
				t_ys = {}
				y_min = self.continuation[self.LIMIT_CYCLE_CURVE].curve[0:len_curve - 1, i]
				y_max = self.continuation[self.LIMIT_CYCLE_CURVE].curve[0:len_curve - 1, i + nb_variables]
				y_min = [y_i for x_i, y_i in zip(x, y_min) if self.fromValue <= x_i <= self.toValue]
				y_max = [y_i for x_i, y_i in zip(x, y_max) if self.fromValue <= x_i <= self.toValue]
				t_ys.update({'min': y_min})
				t_ys.update({'max': y_max})

				curves.update({var: t_ys})

		return x, curves

	def buildPoints(self):

		self.points = {}
		for var in self.continuation[self.MAIN_CURVE].varslist:
			self.points.update({var: []})

		for points_type, points in list(self.continuation[self.MAIN_CURVE].BifPoints.items()):
			for point in points.found:
				if self.fromValue <= point.X[self.parameter.getSymbolStr()] <= self.toValue:
					points_x = point.X[self.parameter.getSymbolStr()]
					for var in self.continuation[self.MAIN_CURVE].varslist:
						self.points[var].append((points_type, points_x, point.X[var]))

	def getPoints(self):

		if self.status == self.SUCCESS:

			return self.points

	def hasHopfBifurcations(self):
		return len(self.continuation[self.MAIN_CURVE].BifPoints['H'].found) > 0

	def findLimitCycleCurves(self, point='H1'):

		if self.hasHopfBifurcations():

			limit_cycle_args = args(name=self.LIMIT_CYCLE_CURVE, type='LC-C')
			limit_cycle_args.initpoint = self.MAIN_CURVE + ':' + point
			limit_cycle_args.freepars = [self.parameter.getSymbolStr()]
			limit_cycle_args.MaxNumPoints = self.maxSteps
			limit_cycle_args.setSize = self.ds
			limit_cycle_args.MinStepSize = self.ds/100
			limit_cycle_args.MaxStepSize = self.ds*100
			limit_cycle_args.LocBifPoints = 'all'
			limit_cycle_args.SaveEigen = True
			limit_cycle_args.verbosity = self.verbosity
			self.continuation.newCurve(limit_cycle_args)


			self.continuation[self.LIMIT_CYCLE_CURVE].forward()

	def plotCurve(self, plot, variable):

		x, ys, stab = self.getStabilitySlicedCurves()
		points = self.getPoints()

		for i, (slice_x, slice_y) in enumerate(zip(x, ys[variable.getSymbolStr()])):

			if stab[i] == 'S':
				color = "b-"
			elif stab[i] == 'N':
				color = 'r--'
			elif stab[i] == 'U':
				color = 'b--'
			else:
				color = 'r-'

			plot.plot(slice_x, slice_y, color)

		points_var = points[variable.getSymbolStr()]
		x = [x_i for _, x_i, _ in points_var]
		y = [y_i for _, _, y_i in points_var]
		plot.plot(x, y, 'ro')
		plot.set_xlim(self.fromValue, self.toValue)

	def plotLimitCycles(self, plot, variable):

		if self.hasHopfBifurcations():
			if not self.LIMIT_CYCLE_CURVE in list(self.continuation.curves.keys()):
				self.findLimitCycleCurves()

			x, ys = self.getLimitCycleCurves()
			plot.plot(x, ys[variable.getSymbolStr()]['min'], 'b-')
			plot.plot(x, ys[variable.getSymbolStr()]['max'], 'b-')


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made for 'high level' tests, using various components

"""
from __future__ import division

from past.utils import old_div
from libsignetsim import SedmlDocument, Settings
from unittest import TestCase
from os.path import join, dirname, isdir
from os import mkdir


class TestSteadyStatesScan(TestCase):
	""" Tests high level functions """


	def testFig4BPubli(self):

		testfiles_path = join(dirname(__file__), "files")

		sedml_doc = SedmlDocument()

		simulations = []

		simulation = sedml_doc.listOfSimulations.createUniformTimeCourse()
		simulation.setInitialTime(0)
		simulation.setOutputStartTime(0)
		simulation.setOutputEndTime(100)
		simulation.setNumberOfPoints(5)
		simulation.getAlgorithm().setCVODE()
		simulation.getAlgorithm().listOfAlgorithmParameters.setRelTol(1e-7)
		simulation.getAlgorithm().listOfAlgorithmParameters.setAbsTol(1e-30)

		model = sedml_doc.listOfModels.createModel()
		model.setLanguageSbml()
		model.setSource("urn:miriam:biomodels.db:BIOMD0000000001")
		sbml_model = model.getSbmlModel()

		change = model.listOfChanges.createChangeAttribute()
		change.getTarget().setModelObject(sbml_model.listOfParameters.getBySbmlId("t2"), attribute="value")
		change.setNewValue(100.0)

		task = sedml_doc.listOfTasks.createTask()
		task.setModel(model)
		task.setSimulation(simulation)

		data_time = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_time.setName("Time")
		var_time = data_time.listOfVariables.createVariable("time")
		var_time.setTask(task)
		var_time.setModel(model)
		var_time.setSymbolTime()
		data_time.getMath().setInternalMathFormula(var_time.getSympySymbol())

		# Let's try to first reproduce the Basal on the figure, aka the fraction of total receptors
		# So we need all vars to compute the fraction
		data_basal = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_basal.setName("Basal")

		data_basal_var_basal = data_basal.listOfVariables.createVariable("B")
		data_basal_var_basal.setTask(task)
		data_basal_var_basal.setModel(model)
		data_basal_var_basal.setTarget(sbml_model.listOfSpecies.getBySbmlId("B"))

		data_basal_var_basalAch = data_basal.listOfVariables.createVariable("BL")
		data_basal_var_basalAch.setTask(task)
		data_basal_var_basalAch.setModel(model)
		data_basal_var_basalAch.setTarget(sbml_model.listOfSpecies.getBySbmlId("BL"))

		data_basal_var_DLL = data_basal.listOfVariables.createVariable("DLL")
		data_basal_var_DLL.setTask(task)
		data_basal_var_DLL.setModel(model)
		data_basal_var_DLL.setTarget(sbml_model.listOfSpecies.getBySbmlId("DLL"))

		data_basal_var_ILL = data_basal.listOfVariables.createVariable("ILL")
		data_basal_var_ILL.setTask(task)
		data_basal_var_ILL.setModel(model)
		data_basal_var_ILL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ILL"))

		data_basal_var_ALL = data_basal.listOfVariables.createVariable("ALL")
		data_basal_var_ALL.setTask(task)
		data_basal_var_ALL.setModel(model)
		data_basal_var_ALL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ALL"))

		t_formula = old_div(data_basal_var_basal.getSympySymbol(), (
			data_basal_var_basal.getSympySymbol()
			+ data_basal_var_basalAch.getSympySymbol()
			+ data_basal_var_DLL.getSympySymbol()
			+ data_basal_var_ILL.getSympySymbol()
			+ data_basal_var_ALL.getSympySymbol()
		))
		data_basal.getMath().setInternalMathFormula(t_formula)


		data_basalAch = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_basalAch.setName("BasalACh")

		data_basalAch_var_basal = data_basalAch.listOfVariables.createVariable("B")
		data_basalAch_var_basal.setTask(task)
		data_basalAch_var_basal.setModel(model)
		data_basalAch_var_basal.setTarget(sbml_model.listOfSpecies.getBySbmlId("B"))

		data_basalAch_var_basalAch = data_basalAch.listOfVariables.createVariable("BL")
		data_basalAch_var_basalAch.setTask(task)
		data_basalAch_var_basalAch.setModel(model)
		data_basalAch_var_basalAch.setTarget(sbml_model.listOfSpecies.getBySbmlId("BL"))

		data_basalAch_var_DLL = data_basalAch.listOfVariables.createVariable("DLL")
		data_basalAch_var_DLL.setTask(task)
		data_basalAch_var_DLL.setModel(model)
		data_basalAch_var_DLL.setTarget(sbml_model.listOfSpecies.getBySbmlId("DLL"))

		data_basalAch_var_ILL = data_basalAch.listOfVariables.createVariable("ILL")
		data_basalAch_var_ILL.setTask(task)
		data_basalAch_var_ILL.setModel(model)
		data_basalAch_var_ILL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ILL"))

		data_basalAch_var_ALL = data_basalAch.listOfVariables.createVariable("ALL")
		data_basalAch_var_ALL.setTask(task)
		data_basalAch_var_ALL.setModel(model)
		data_basalAch_var_ALL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ALL"))

		t_formula = old_div(data_basalAch_var_basalAch.getSympySymbol(), (
			data_basalAch_var_basal.getSympySymbol()
			+ data_basalAch_var_basalAch.getSympySymbol()
			+ data_basalAch_var_DLL.getSympySymbol()
			+ data_basalAch_var_ILL.getSympySymbol()
			+ data_basalAch_var_ALL.getSympySymbol()
		))
		data_basalAch.getMath().setInternalMathFormula(t_formula)


		data_DLL = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_DLL.setName("DesensitisedACh2")

		data_DLL_var_basal = data_DLL.listOfVariables.createVariable("B")
		data_DLL_var_basal.setTask(task)
		data_DLL_var_basal.setModel(model)
		data_DLL_var_basal.setTarget(sbml_model.listOfSpecies.getBySbmlId("B"))

		data_DLL_var_basalAch = data_DLL.listOfVariables.createVariable("BL")
		data_DLL_var_basalAch.setTask(task)
		data_DLL_var_basalAch.setModel(model)
		data_DLL_var_basalAch.setTarget(sbml_model.listOfSpecies.getBySbmlId("BL"))

		data_DLL_var_DLL = data_DLL.listOfVariables.createVariable("DLL")
		data_DLL_var_DLL.setTask(task)
		data_DLL_var_DLL.setModel(model)
		data_DLL_var_DLL.setTarget(sbml_model.listOfSpecies.getBySbmlId("DLL"))

		data_DLL_var_ILL = data_DLL.listOfVariables.createVariable("ILL")
		data_DLL_var_ILL.setTask(task)
		data_DLL_var_ILL.setModel(model)
		data_DLL_var_ILL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ILL"))

		data_DLL_var_ALL = data_DLL.listOfVariables.createVariable("ALL")
		data_DLL_var_ALL.setTask(task)
		data_DLL_var_ALL.setModel(model)
		data_DLL_var_ALL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ALL"))

		t_formula = old_div(data_DLL_var_DLL.getSympySymbol(), (
			data_DLL_var_basal.getSympySymbol()
			+ data_DLL_var_basalAch.getSympySymbol()
			+ data_DLL_var_DLL.getSympySymbol()
			+ data_DLL_var_ILL.getSympySymbol()
			+ data_DLL_var_ALL.getSympySymbol()
		))
		data_DLL.getMath().setInternalMathFormula(t_formula)


		data_ILL = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_ILL.setName("IntermediateACh2")

		data_ILL_var_basal = data_ILL.listOfVariables.createVariable("B")
		data_ILL_var_basal.setTask(task)
		data_ILL_var_basal.setModel(model)
		data_ILL_var_basal.setTarget(sbml_model.listOfSpecies.getBySbmlId("B"))

		data_ILL_var_basalAch = data_ILL.listOfVariables.createVariable("BL")
		data_ILL_var_basalAch.setTask(task)
		data_ILL_var_basalAch.setModel(model)
		data_ILL_var_basalAch.setTarget(sbml_model.listOfSpecies.getBySbmlId("BL"))

		data_ILL_var_DLL = data_ILL.listOfVariables.createVariable("DLL")
		data_ILL_var_DLL.setTask(task)
		data_ILL_var_DLL.setModel(model)
		data_ILL_var_DLL.setTarget(sbml_model.listOfSpecies.getBySbmlId("DLL"))

		data_ILL_var_ILL = data_ILL.listOfVariables.createVariable("ILL")
		data_ILL_var_ILL.setTask(task)
		data_ILL_var_ILL.setModel(model)
		data_ILL_var_ILL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ILL"))

		data_ILL_var_ALL = data_ILL.listOfVariables.createVariable("ALL")
		data_ILL_var_ALL.setTask(task)
		data_ILL_var_ALL.setModel(model)
		data_ILL_var_ALL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ALL"))

		t_formula = old_div(data_ILL_var_ILL.getSympySymbol(), (
			data_ILL_var_basal.getSympySymbol()
			+ data_ILL_var_basalAch.getSympySymbol()
			+ data_ILL_var_DLL.getSympySymbol()
			+ data_ILL_var_ILL.getSympySymbol()
			+ data_ILL_var_ALL.getSympySymbol()
		))
		data_ILL.getMath().setInternalMathFormula(t_formula)


		data_ALL = sedml_doc.listOfDataGenerators.createDataGenerator()
		data_ALL.setName("ActiveACh2")

		data_ALL_var_basal = data_ALL.listOfVariables.createVariable("B")
		data_ALL_var_basal.setTask(task)
		data_ALL_var_basal.setModel(model)
		data_ALL_var_basal.setTarget(sbml_model.listOfSpecies.getBySbmlId("B"))

		data_ALL_var_basalAch = data_ALL.listOfVariables.createVariable("BL")
		data_ALL_var_basalAch.setTask(task)
		data_ALL_var_basalAch.setModel(model)
		data_ALL_var_basalAch.setTarget(sbml_model.listOfSpecies.getBySbmlId("BL"))

		data_ALL_var_DLL = data_ALL.listOfVariables.createVariable("DLL")
		data_ALL_var_DLL.setTask(task)
		data_ALL_var_DLL.setModel(model)
		data_ALL_var_DLL.setTarget(sbml_model.listOfSpecies.getBySbmlId("DLL"))

		data_ALL_var_ILL = data_ALL.listOfVariables.createVariable("ILL")
		data_ALL_var_ILL.setTask(task)
		data_ALL_var_ILL.setModel(model)
		data_ALL_var_ILL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ILL"))

		data_ALL_var_ALL = data_ALL.listOfVariables.createVariable("ALL")
		data_ALL_var_ALL.setTask(task)
		data_ALL_var_ALL.setModel(model)
		data_ALL_var_ALL.setTarget(sbml_model.listOfSpecies.getBySbmlId("ALL"))

		t_formula = old_div(data_ALL_var_ALL.getSympySymbol(), (
			data_ALL_var_basal.getSympySymbol()
			+ data_ALL_var_basalAch.getSympySymbol()
			+ data_ALL_var_DLL.getSympySymbol()
			+ data_ALL_var_ILL.getSympySymbol()
			+ data_ALL_var_ALL.getSympySymbol()
		))
		data_ALL.getMath().setInternalMathFormula(t_formula)



		plot = sedml_doc.listOfOutputs.createPlot2D()
		curve_basal = plot.listOfCurves.createCurve()
		curve_basal.setXData(data_time)
		curve_basal.setYData(data_basal)
		curve_basal.setLogX(True)

		curve_basalAch = plot.listOfCurves.createCurve()
		curve_basalAch.setXData(data_time)
		curve_basalAch.setYData(data_basalAch)
		curve_basalAch.setLogX(True)

		curve_dll = plot.listOfCurves.createCurve()
		curve_dll.setXData(data_time)
		curve_dll.setYData(data_DLL)
		curve_dll.setLogX(True)

		curve_ill = plot.listOfCurves.createCurve()
		curve_ill.setXData(data_time)
		curve_ill.setYData(data_ILL)
		curve_ill.setLogX(True)

		curve_all = plot.listOfCurves.createCurve()
		curve_all.setXData(data_time)
		curve_all.setYData(data_ALL)
		curve_all.setLogX(True)


		report = sedml_doc.listOfOutputs.createReport()

		dataset_time = report.listOfDataSets.createDataSet()
		dataset_time.setLabel("Time")
		dataset_time.setData(data_time)

		dataset_basal = report.listOfDataSets.createDataSet()
		dataset_basal.setLabel("Basal")
		dataset_basal.setData(data_basal)

		dataset_basalAch = report.listOfDataSets.createDataSet()
		dataset_basalAch.setLabel("BasalACh")
		dataset_basalAch.setData(data_basalAch)

		dataset_DLL = report.listOfDataSets.createDataSet()
		dataset_DLL.setLabel("DesensitisedACh2")
		dataset_DLL.setData(data_DLL)

		dataset_ILL = report.listOfDataSets.createDataSet()
		dataset_ILL.setLabel("IntermediateACh2")
		dataset_ILL.setData(data_ILL)

		dataset_ALL = report.listOfDataSets.createDataSet()
		dataset_ALL.setLabel("ActiveACh2")
		dataset_ALL.setData(data_ALL)
		sedml_doc.run()

		# expected_data = {
		# 	"Basal": [1.0, 0.8703929751604004, 0.7749423419177442, 0.6946061123069424, 0.597290529625008, 0.4537850318551511,
		#  0.34750139692209975, 0.29949244626945737, 0.22270499980202377, 0.10951146314675342, 0.03412836890172797,
		#  0.024200752996025095, 0.022706292875704824, 0.019314868585969504, 0.012743352308043987, 0.004557742803571135,
		#  0.000838864752742402],
		# 	"BasalACh": [0.0, 0.12733989706088109, 0.20860665566561312, 0.23515925558680748, 0.208278208041077, 0.1632517601817593,
		#  0.12918341396276756, 0.11193037804649376, 0.08324269397045204, 0.04095051744993127, 0.012785114865493191,
		#  0.009075094010791105, 0.00851472322023259, 0.007242960070971262, 0.004778682083793607, 0.0017091291129665094,
		#  0.00031457282942824776],
		# 	"DesensitisedACh2": [0.0, 5.276504575778524e-13, 3.331032690025042e-11, 1.3348713563479085e-09, 3.390793915547514e-08,
		#  5.913794319390883e-07, 7.374447747054566e-06, 6.638876496189277e-05, 0.00046273880818307436,
		#  0.0025754514927221668, 0.010801340789850395, 0.034193460998989275, 0.09266196884291755, 0.22816586990710194,
		#  0.49072979203151695, 0.8177845618016254, 0.9663717628010179],
		# 	"IntermediateACh2": [0.0, 7.928585437579911e-07, 1.7394771632477156e-05, 0.00023455456576278516, 0.002016484487472414,
		#  0.012106619410986482, 0.05107905183405494, 0.15467605453139202, 0.3705004603739618, 0.68727111950252,
		#  0.8914220977379401, 0.8960358851611324, 0.8418739633309424, 0.7161477843260169, 0.4725300197675206,
		#  0.16907495528569005, 0.031209549157480712],
		# 	"ActiveACh2": [0.0, 0.0022663349196470604, 0.016433607611699937, 0.070000076205616, 0.1924147439385037, 0.3708559971726711,
		#  0.47222876283333054, 0.43383473238769515, 0.32308910704537935, 0.15969144840807312, 0.050863077704988394,
		#  0.03649480683306223, 0.034243051730202576, 0.029128517109940463, 0.019218153809124724, 0.006873610996146915,
		#  0.0012652504593307923]
		# }
		#
		# simulated_data = sedml_doc.listOfOutputs.getReports()[0].getData()
		#
		#
		# for var, values in expected_data.items():
		# 	if var != "Time":
		# 		for i, value in enumerate(values):
		# 			self.assertAlmostEqual(value, simulated_data[var][i])

		sedml_doc.writeSedmlToFile(join(Settings.tempDirectory, "BIOMD0000000001_fig4b.xml"))
		sedml_doc = SedmlDocument()
		sedml_doc.readSedmlFromFile(join(Settings.tempDirectory, "BIOMD0000000001_fig4b.xml"))
		sedml_doc.run()
	#
	# def testFig4CPubli(self):
	# 	""" Here the tricky part is that we must integrate the first 20 seconds,
	# 		until the event (with whatever timescale I guess),
	# 		and then integrate 100s with a log scale
	#
	# 	 	Maybe and idea is to have a repeated task, containing :
	# 	 	- a simulation for the first 20s, with just two time points
	# 	 	- a repeated task, containing the log scale simulation
	#
	# 	 	Should work, but not sure how to just plot discarding the first simulation
	# 	"""
	# 	testfiles_path = join(dirname(__file__), "files")
	# 	if not isdir(testfiles_path):
	# 		mkdir(testfiles_path)
	#
	# 	sedml_doc = SedmlDocument()
	#
	#
	# 	simulation_init = sedml_doc.listOfSimulations.createUniformTimeCourse()
	# 	simulation_init.setInitialTime(0)
	# 	simulation_init.setOutputStartTime(20)
	# 	simulation_init.setOutputEndTime(20)
	# 	simulation_init.setNumberOfPoints(1)
	#
	# 	simulation_onestep = sedml_doc.listOfSimulations.createOneStep()
	# 	simulation_onestep.getAlgorithm().setCVODE()
	# 	simulation_onestep.getAlgorithm().listOfAlgorithmParameters.setRelTol(1e-6)
	# 	simulation_onestep.getAlgorithm().listOfAlgorithmParameters.setAbsTol(1e-30)
	#
	#
	# 	model = sedml_doc.listOfModels.createModel()
	# 	model.setLanguageSbml()
	# 	model.setSource("urn:miriam:biomodels.db:BIOMD0000000001")
	# 	sbml_model = model.getSbmlModel()
	#
	# 	task_init = sedml_doc.listOfTasks.createTask()
	# 	task_init.setModel(model)
	# 	task_init.setSimulation(simulation_init)
	#
	# 	task_onestep = sedml_doc.listOfTasks.createTask()
	# 	task_onestep.setModel(model)
	# 	task_onestep.setSimulation(simulation_onestep)
	#
	# 	task_logscale = sedml_doc.listOfTasks.createRepeatedTask()
	# 	task_logscale.setResetModel(False)
	#
	# 	uniform_range = task_logscale.listOfRanges.createUniformRange()
	# 	uniform_range.setStart(6e-5)
	# 	uniform_range.setEnd(1e+2)
	# 	uniform_range.setNumberOfPoints(15)
	# 	uniform_range.setLog()
	# 	task_logscale.setRange(uniform_range)
	#
	# 	sub_task_onestep = task_logscale.listOfSubTasks.createSubTask()
	# 	sub_task_onestep.setTask(task_onestep)
	# 	sub_task_onestep.setOrder(1)
	#
	# 	master_task = sedml_doc.listOfTasks.createRepeatedTask()
	# 	master_task.setResetModel(False)
	#
	# 	sub_task_init = master_task.listOfSubTasks.createSubTask()
	# 	sub_task_init.setTask(task_init)
	# 	sub_task_init.setOrder(1)
	#
	# 	sub_task_log = master_task.listOfSubTasks.createSubTask()
	# 	sub_task_log.setTask(task_logscale)
	# 	sub_task_log.setOrder(2)
	#
	#
	# 	data_time = sedml_doc.listOfDataGenerators.createDataGenerator()
	# 	data_time.setName("Time")
	# 	var_time = data_time.listOfVariables.createVariable("time")
	# 	var_time.setTask(master_task)
	# 	var_time.setModel(model)
	# 	var_time.setSymbolTime()
	# 	data_time.getMath().setInternalMathFormula(var_time.getSympySymbol())
	#
	#
	#
	# 	# Let's try to first reproduce the Basal on the figure, aka the fraction of total receptors
	# 	# So we need all vars to compute the fraction
	# 	data_basal = sedml_doc.listOfDataGenerators.createDataGenerator()
	# 	data_basal.setName("Basal")
	#
	#
	# 	data_basal_var_basal = data_basal.listOfVariables.createVariable("B")
	# 	data_basal_var_basal.setTask(master_task)
	# 	data_basal_var_basal.setModel(model)
	# 	data_basal_var_basal.setTarget(sbml_model.listOfSpecies.getBySbmlId("B"))
	#
	# 	data_basal_var_intermediate = data_basal.listOfVariables.createVariable("I")
	# 	data_basal_var_intermediate.setTask(master_task)
	# 	data_basal_var_intermediate.setModel(model)
	# 	data_basal_var_intermediate.setTarget(sbml_model.listOfSpecies.getBySbmlId("I"))
	#
	# 	data_basal_var_desensitised = data_basal.listOfVariables.createVariable("D")
	# 	data_basal_var_desensitised.setTask(master_task)
	# 	data_basal_var_desensitised.setModel(model)
	# 	data_basal_var_desensitised.setTarget(sbml_model.listOfSpecies.getBySbmlId("D"))
	#
	#
	# 	data_basal_var_intermediateAch = data_basal.listOfVariables.createVariable("IL")
	# 	data_basal_var_intermediateAch.setTask(master_task)
	# 	data_basal_var_intermediateAch.setModel(model)
	# 	data_basal_var_intermediateAch.setTarget(sbml_model.listOfSpecies.getBySbmlId("IL"))
	#
	# 	data_basal_var_desensitisedAch = data_basal.listOfVariables.createVariable("DL")
	# 	data_basal_var_desensitisedAch.setTask(master_task)
	# 	data_basal_var_desensitisedAch.setModel(model)
	# 	data_basal_var_desensitisedAch.setTarget(sbml_model.listOfSpecies.getBySbmlId("DL"))
	#
	#
	# 	data_basal_var_desensitisedAch2 = data_basal.listOfVariables.createVariable("DLL")
	# 	data_basal_var_desensitisedAch2.setTask(master_task)
	# 	data_basal_var_desensitisedAch2.setModel(model)
	# 	data_basal_var_desensitisedAch2.setTarget(sbml_model.listOfSpecies.getBySbmlId("DLL"))
	#
	# 	data_basal_var_intermediateAch2 = data_basal.listOfVariables.createVariable("ILL")
	# 	data_basal_var_intermediateAch2.setTask(master_task)
	# 	data_basal_var_intermediateAch2.setModel(model)
	# 	data_basal_var_intermediateAch2.setTarget(sbml_model.listOfSpecies.getBySbmlId("ILL"))
	#
	# 	data_basal_var_activeAch2 = data_basal.listOfVariables.createVariable("ALL")
	# 	data_basal_var_activeAch2.setTask(master_task)
	# 	data_basal_var_activeAch2.setModel(model)
	# 	data_basal_var_activeAch2.setTarget(sbml_model.listOfSpecies.getBySbmlId("ALL"))
	#
	#
	# 	t_formula = data_basal_var_basal.getSympySymbol() / (
	# 		data_basal_var_basal.getSympySymbol()
	# 		+ data_basal_var_intermediate.getSympySymbol()
	# 		+ data_basal_var_desensitised.getSympySymbol()
	# 		+ data_basal_var_intermediateAch.getSympySymbol()
	# 		+ data_basal_var_desensitisedAch.getSympySymbol()
	# 		+ data_basal_var_desensitisedAch2.getSympySymbol()
	# 		+ data_basal_var_intermediateAch2.getSympySymbol()
	# 		+ data_basal_var_activeAch2.getSympySymbol()
	# 	)
	# 	data_basal.getMath().setInternalMathFormula(t_formula)
	#
	# 	report = sedml_doc.listOfOutputs.createReport()
	#
	# 	dataset_time = report.listOfDataSets.createDataSet()
	# 	dataset_time.setLabel("Time")
	# 	dataset_time.setData(data_time)
	#
	# 	dataset_basal = report.listOfDataSets.createDataSet()
	# 	dataset_basal.setLabel("Basal")
	# 	dataset_basal.setData(data_basal)
	#
	# 	# sedml_doc.run()
	# 	# simulated_data = sedml_doc.listOfOutputs.getReports()[0].getData()
	#
	# 	sedml_doc.writeSedmlToFile(join(testfiles_path, "BIOMD0000000001_fig4c.xml"))
	# 	sedml_doc = SedmlDocument()
	# 	sedml_doc.readSedmlFromFile(join(testfiles_path, "BIOMD0000000001_fig4c.xml"))
	#
		pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase
from libsignetsim.settings.Settings import Settings


class ListOfIds(list):

	def __init__(self, document):

		list.__init__(self)
		self.__document = document

	def ids(self):
		""" Return a set of ids of the sedml objects """
		return [obj.getId() for obj in self]

	def getById(self, sedml_id, pos=0):
		""" Find sedml objects by their Id """

		res = []
		for obj in self:
			if obj.getId() == sedml_id:
				res.append(obj)

		if len(res) > 0:
			return res[pos]

	def containsId(self, sedml_id):
		""" Test if an sbml id is in the list """

		res = False
		for obj in self:
			if sedml_id == obj.getId():
				res = True

		return res
